%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino
% This program estimates the TV FAVAR
%==========================================================================


%=== Volatility options
% vola_fun = 0;  % 'Baseline: affine function of past observables';  
% vola_fun = 1 % 'Baseline: affine function of squared past observables';    
% vola_fun = 0 % 'Baseline: affine function of contemporaneous observables';    
% vola_fun = 3 % 'Baseline: affine function of squared latent factors and FCI^2';    
% vola_fun = 4 % 'Baseline: affine function and uncertainty index added to set of regressors';            

clear all;
tic;
cd ../Results;

global opt_starttime
%=== Load Baseline specification
load fMatyMat_Baseline_FCI_Watson_nbfac10_G9

%=== Load GARCH specification
%load fMatyMat_GARCH_FCI_Watson_nbfac10_G9

%=== Load specification where FCI is purged from US macro variables
%load fMatyMat_Baseline_FCI_Watson_purged_nbfac10_G9

%=== Load specification with International FCI
%load fMatyMat_Baseline_FCI_International_nbfac10_G9

%=== Load specification with Hubrich and Tetlow FSI
%load fMatyMat_Baseline_FSI_nbfac10_G9

cd ../CodeData;
 

%Determining the nb of factors:
kmax = 20;
% disp('Info crit. applied to the Y  ')
ncmat = nbf_infobaing(yMat,kmax,0);  %458

 
%____________________________________________________________________________
%==== Select functional form for volatility
wanttvvola = 1;   % 1: version with tv factor vola, 0: version with constant vola
volavar = 1;      % if =0: Lagged fac, if =1: obs vola, (others ,eg  lagged differences to be added)

disp(['Volatility option: ', num2str(volatype)])
disp(['Volatility option: ', num2str(vola_fun)])
disp(['FCI exogenous? ', num2str(wantblockexogobs)])
 
wantARidio = 1; % NEWAR2: set as 1 or 2 or 0
p = 1; %2
%______________________________________________________
%   Extract key info params from Data and put settings for estimation
%______________________________________________________
[~,N]=size(yMat);

if wantblockexogobs==1;
    IdentStrg=[IdentStrg,'_ObsExog','_p',num2str(p)];
else
    IdentStrg=[IdentStrg,'_ObsEndog','_p',num2str(p)];
    
end;

if wanttvvola==1;
    if volavar==0;
        IdentStrg=[IdentStrg,'_VolaLagFac'];    
    elseif volavar==1;
        IdentStrg=[IdentStrg,'_VolaObs'];
    end;
    if volatype==0;
        IdentStrg=[IdentStrg,'_ExpAff']; 
        
    elseif volatype==2; % baseline
        IdentStrg=[IdentStrg,'_AffLev'];
        % 1971Q2-2012Q4
        
        if strcmp(opt_starttime,'1971')
            if vola_fun ==2
                vola_range = 'b8:aa173';
            else
                vola_range = 'b7:aa172';
            end
        elseif strcmp(opt_starttime,'1973')
            if vola_fun ==2
                vola_range = 'b16:aa173';
            else
                vola_range = 'b15:aa172';
            end
        elseif strcmp(opt_starttime,'1975')
            if vola_fun ==2
                vola_range = 'b24:aa172';
            else
                vola_range = 'b23:aa172';
            end
        end

        
        if vola_fun ==0
            
            disp('Baseline: affine function of past observables')
            ObsVolaLag1 = xlsread('WL_ObsVola.xls','QuarterlyVola',vola_range); % this is already lagged by one quarter
            ObsVolaLag1 = ObsVolaLag1(:,[12,13,18]);
            
            
        elseif vola_fun == 1; % use squared past observables
            disp('Baseline: affine function of squared past observables')
            IdentStrg=[IdentStrg,'_AffSq'];
            ObsVolaLag1 = xlsread('WL_ObsVola.xls','QuarterlyVola',vola_range); % this is already lagged by one quarter
            ObsVolaLag1 = ObsVolaLag1(:,[12,13,18]).^2;
            
        elseif  vola_fun == 2; % use contemp. observables
            disp('Baseline: affine function of contemporaneous observables')
            IdentStrg=[IdentStrg,'_AffCont'];
            ObsVolaLag1 = xlsread('WL_ObsVola.xls','QuarterlyVola',vola_range); % contemp
            ObsVolaLag1 = ObsVolaLag1(:,[12,13,18]);
            
        elseif  vola_fun == 3; % use squared factors
            disp('Baseline: affine function of squared factors')
            IdentStrg=[IdentStrg,'_AffSqR'];
            ObsVolaLag1 = [NaN(1,size(fMat,2)); fMat(1:end-1,:).^2]; %squared past factors
            
        elseif vola_fun ==4 % Add uncertainty measure to the set of regressors
            ObsVolaLag1 = xlsread('WL_ObsVola.xls','QuarterlyVola',vola_range); % this is already lagged by one quarter
            ObsVolaLag1 = ObsVolaLag1(:,[12,13,18,19]);
            IdentStrg=[IdentStrg,'_UNC'];
        end
        
    elseif volatype==5;
        IdentStrg=[IdentStrg,'_GARCH'];
    end;
else
    IdentStrg=[IdentStrg,'_VolaConst'];
end;

[T,G]=size(fMat);
P0Scale=1;
epstol=1e-7;
optalgo=2;

%______________________________________________________
%   Containers for estimated tv  parameters
%______________________________________________________

% I streamlined the tv VAR estimation, so need containers here only for the cross-sectional
% equations

RvecEstFornontv=NaN(T,1);
nontvLamb=NaN(N,G);

tvLamb=NaN(T,N,G);
tvLambSmo=NaN(T,N,G);
lamb0Fortv=zeros(N,1);
RvecEstFortv=NaN(N,1); % stddevs of idiosync  factor comps
%rhovecEstFortv=NaN(N,1);
if wantARidio>0 %NEWAR2:
    rhovecEstFortv=NaN(N,wantARidio);
end;

%%
%______________________________________________________
%   Estimate tv VAR, equation-wise
%______________________________________________________
% Impose the restrictions
% the 'Types' are 
% 0: non-tv param, pre-set, 
% 1: non-tv param, to be estimated
% 2: tv-param

% The autoregr matrices
KapType=NaN(G,G,p);
KapFix=NaN(G,G,p);

for pp=1:1;
    KapType_pp=2*ones(G,G);
    if wantblockexogobs==1;
        KapType_pp(1:nbobs,nbobs+1:end)=0;
    end;
    KapFix_pp=NaN(G,G);
    if wantblockexogobs==1;
        KapFix_pp(1:nbobs,nbobs+1:end)=0;
    end;
    KapType(:,:,pp)=KapType_pp;
    KapFix(:,:,pp)=KapFix_pp;
end;
if p>1;
for pp=2:p;
    for gg=nbobs+1:G;
    KapType_pp(gg,1:end)=2*ones(1,G);
    KapFix_pp(gg,1:end)=NaN(1,G);
%     for gg=1:G;
%         KapType_pp(gg,gg)=1;
%         KapFix_pp(gg,gg)=NaN;
%     end;
    end;
    for gg=1:nbobs;
    KapType_pp(gg,1:end)=2*ones(1,G);
    KapFix_pp(gg,1:end)=NaN(1,G);
%     for gg=1:G;
%         KapType_pp(gg,gg)=1;
%         KapFix_pp(gg,gg)=NaN;
%     end;
    end;
    if wantblockexogobs==1;
        KapType_pp(1:nbobs,nbobs+1:end)=0;
    end;
    
    if wantblockexogobs==1;
        KapFix_pp(1:nbobs,nbobs+1:end)=0;
    end;
    KapType(:,:,pp)=KapType_pp;
    KapFix(:,:,pp)=KapFix_pp;
end;
end;
% The upper-triang part of P is always the upper-triang part of the id.
% matrix
PType=eye(G);
for gg=2:G;
     PType(gg,1:gg-1)=2;
%     PType(gg,1:gg-1)=1;%2;
end;
PFix=NaN(G,G);
%IdentStrg=[IdentStrg,'_PType1'];


% the vola regressors
ZZ=cell(G,1);



% Fill the vola regressors
if wanttvvola==1;
    if volavar==0;
        if wantblockexogobs==1;
            for gg=1:nbobs; %
                ZZ{gg}=[[NaN(1,1); ones(T-1,1)] [NaN(1,nbobs); fMat(1:end-1,1:nbobs)]];
            end;
            for gg=nbobs+1:G;
                ZZ{gg}=[[NaN(1,1); ones(T-1,1)] [NaN(1,G); fMat(1:end-1,1:G)]];
            end;
        else
            for gg=1:G; %
                ZZ{gg}=[[NaN(1,1); ones(T-1,1)] [NaN(1,G); fMat(1:end-1,1:G)]];
            end;
        end;
    elseif volavar==1 && not(volatype == 5);
        for gg=1:G; %
            ZZ{gg}=[ones(T,1) ObsVolaLag1];
        end;
    end;
else
    for gg=1:G; %
        ZZ{gg}=[NaN(1,1); ones(T-1,1)];
    end;
end;


if volatype == 5, ZZ = []; end

%%
% This function runs the actual optimization
tic;
cFix=zeros(G,1);
% Estimates Factor Volatility
% fMat:     Factors
% ZZ:       Volatility Regressors
% volatype: functional form for volatility (5 --> GARCH)
Results_tv_VAR=WL_tvVAR(KapType,PType,KapFix,PFix,cFix,fMat,ZZ,p,P0Scale,epstol,optalgo,volatype);

%Results_tv_VAR.InnovVola gives std of the factor innovations! In baseline Results_tv_VAR.InnovVola(:,1) the FCI factor vola.
x = [];
for gg = 1:G
    x = [x Results_tv_VAR.Hyperpars.khat{gg}];
end
disp(x)
%F?r jede der G Gleichungen sollten wir haben, dass gamma_1+gamma_2 kleiner als 1 ist. 
%Au?erdem ist ?blicherweise gamma_2>gamma_1. Au?erdem sollten sie nicht an den Startwerten kleben bleiben. 
%Verd?chtig ist wenn du gamma_1=0.1 und gamma_2=0.7 erh?ltst. 
disp('Check (i) x(2)+x(3)<1 ? (ii) x(3)>x(2) ? x(2) ~= .1 and x(3) ~= .7  ?')
tstats_GARCH = [];
for gg = 1:G
    %tstats_GARCH = [tstats_GARCH];
    disp(['T-stats for the ' num2str(gg) '-th equation']) 
    disp(Results_tv_VAR.Hyperpars.khat{gg}./Results_tv_VAR.Hyperpars.se_khat{gg})
end

%  estimates and significance of tv variance parameters
khatCells=Results_tv_VAR.Hyperpars.khat;
se_khatCells=Results_tv_VAR.Hyperpars.se_khat;
volapar = []; %gives significance of the vola proxies in equ 2.8 of paper, which we report in the paper
for gg=1:G;
    disp(['Loadings of variance parameters, equ ',num2str(gg)]);
    disp('first is the intercept');
    khat=khatCells{gg};
    se_khat=se_khatCells{gg};
    tval_khat=khat./se_khat;
    myvolapar=khat;
    mytvalvolapar=tval_khat;
    if volatype>0;
        myvolapar=khat.^2;
        mytvalvolapar=0.5*tval_khat;
    end;
    disp('param estimates');
    disp(myvolapar);
    disp('t values');
    disp(mytvalvolapar);
    volapar = [volapar mytvalvolapar]; 
end;


%%
%______________________________________________________
%  Save results of tv VAR
%______________________________________________________
cd ../Results;
 
save(['Res_G_',IdentStrg],'IdentStrg','Results_tv_VAR','-v6');
cd ../CodeData;
%return;

%%
%______________________________________________________
%   Estimate tv loading equations, equationwise
%______________________________________________________
%N=9; %if only want GDP growth results, not the other variables (then faster)
for nn=1:N;
    disp(['Estimate ' num2str(nn) 'th of ' num2str(N) ' Loading equations']);
    pause(2);
    y=yMat(:,nn);
    % XA=ones(T,1); % version with const intercept to be estimated
    XA=[];          % version without intercept 
    XB=fMat(:,:);
    X=[XA XB];
    %tvind=[0 ones(1,G)]; % version with const intercept to be estimated
    tvind=[ones(1,G)]; % version without intercept 
    RWstdini=0.1*ones(sum(tvind),1);
    
    [betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y,X);
    RvecEstFornontv(nn)=sqrt(sum(resid.*resid))/sqrt(length(y)-length(betahat));
        
    % version without intercept
%     a0=betahat(1:end);
%     P0=P0Scale*Varbetahat(1:end,1:end);
    a0=NaN; %NEWAR2
    P0=NaN;
    
    % version with const intercept to be estimated
    % a0=betahat(2:end);
    % P0=P0Scale*Varbetahat(2:end,2:end);
    
    nontvLamb(nn,:)=a0';
    
    %a0=zeros(N,1);
    %P0=eye(sum(tvind));
    P0scaleWithinFunction=1; % =1 to avoid to scale P0 again internally
    nontvini=[];
    if wantARidio==1; % NEWAR2
    % Version with AR1 idio 
        %a0=[a0; 0]; % NEWAR2: comment these out
        %P0sm=P0; P0=zeros(size(P0)+1); P0(1:end-1,1:end-1)=P0sm;
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,parfilt,parsmo,parfiltVari,parsmoVari,Xnew,stdepsini,rhohat,yprs,Fprs]=WL_tvreg_AR1error(y,X,tvind,a0,P0,P0Scale,RWstdini,nontvini,epstol,optalgo);
        rhovecEstFortv(nn,:)=rhohat;
        disp('rhohat'); disp(rhohat)
    elseif wantARidio==2; %NEWAR2:
        % Version with AR2 idio 
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,parfilt,parsmo,parfiltVari,parsmoVari,Xnew,stdepsini,rhohat,yprs,Fprs]=WL_tvreg_AR2error(y,X,tvind,a0,P0,P0Scale,RWstdini,nontvini,epstol,optalgo);
        rhovecEstFortv(nn,:)=rhohat;
        disp('rhohat'); disp(rhohat)
    elseif wantARidio<1; % NEWAR2:
        % Version with white-noise idiosync
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,parfilt,parsmo,parfiltVari,parsmoVari,Xnew,stdepsini,yprs,Fprs]=WL_tvreg(y,X,tvind,a0,P0,P0scaleWithinFunction,RWstdini,nontvini,epstol,optalgo);
        rhovecEstFortv(nn)=0;
    end;
        
    tvLamb(:,nn,:)=parfilt(:,1:G); % for the AR1 idio version, the last state element is the resid innov, which is not saved here
    tvLambSmo(:,nn,:)=parsmo(:,1:G);
    %lamb0Fortv(nn)=nontvparhat(1); % version with const intercept to be
    %estimated
    RvecEstFortv(nn)=stdepshat;
    OneStepPredObsPointFortv(:,nn)=yprs;
    OneStepPredObsVariFortv(:,nn)=Fprs;    
    
end;
Results_tv_Loadings.tvLamb=tvLamb;
Results_tv_Loadings.tvLambSmo=tvLambSmo; 
Results_tv_Loadings.lamb0Fortv=lamb0Fortv;
Results_tv_Loadings.RvecEstFortv=RvecEstFortv;
Results_tv_Loadings.rhovecEstFortv=rhovecEstFortv;
Results_tv_Loadings.OneStepPredObsPointFortv=OneStepPredObsPointFortv;
Results_tv_Loadings.OneStepPredObsVariFortv=OneStepPredObsVariFortv;

disp(volapar) %give the tstats of cst and the 3 observables in the factor vola equations
 


%______________________________________________________
%  Save results
%______________________________________________________
cd ../Results;
if wantARidio==1; %NEWAR2
    IdentStrg=[IdentStrg,'_AR1idio'];
elseif wantARidio==2;
    IdentStrg=[IdentStrg,'_AR2idio'];
end;
save(['Res_N_',IdentStrg],'IdentStrg','Results_tv_Loadings');
cd ../CodeData;
toc;


 