%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino
% This program generates the FEVD
%==========================================================================

 clear all;

%=======================================================================
%=== Load the result files:
tic;
cd ..\Results;
disp('Load: factors and Y, time-varying estimation results');

%=== Work with Baseline specification
% load fMatyMat_Baseline_FCI_Watson_nbfac10_G9 
% load Res_G_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%=== Work with Baseline FCI specification and GARCH
load fMatyMat_GARCH_FCI_Watson_nbfac10_G9 
load Res_G_GARCH_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_GARCH
load Res_N_GARCH_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_GARCH_AR1idio

%=== Work with specification where FCI is purged from US macro variables
% load fMatyMat_Baseline_FCI_Watson_purged_nbfac10_G9
% load Res_G_Baseline_FCI_Watson_purged_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FCI_Watson_purged_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%=== Work with specification with International FCI
% load fMatyMat_Baseline_FCI_International_nbfac10_G9
% load Res_G_Baseline_FCI_International_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FCI_International_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%=== Work with FSI specification
% load fMatyMat_Baseline_FSI_nbfac10_G9
% load Res_G_Baseline_FSI_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FSI_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

cd ..\CodeData; 
%=======================================================================
 
%=== SET these parameters 
FCIPos = pos_fci; % Position of FCI
Hor=20;
cutl=0;
Hor=20;
cutl=0;


%=== Select the first 9 variables  (GDP)
DispVar_nv = [1:9]';
[nbCtr,nbVar]=size(DispVar_nv);

%=== Identify the individual result arrays/matrices:
tvKap=Results_tv_VAR.RestrSmo.tvKap;
tvP=Results_tv_VAR.RestrSmo.tvP;
tvKapTildeComp=Results_tv_VAR.RestrSmo.tvKapTildeComp;
tvKapTildeCompMaxEig=Results_tv_VAR.RestrSmo.tvKapTildeCompMaxEig;
InnovVola=Results_tv_VAR.InnovVola;

tvLamb=Results_tv_Loadings.tvLambSmo;
lamb0Fortv=Results_tv_Loadings.lamb0Fortv;
RvecEstFortv=Results_tv_Loadings.RvecEstFortv;
rhovecEstFortv=Results_tv_Loadings.rhovecEstFortv;

xtime=((str2double(opt_starttime)+2/4):(1/4):(2012+3/4-cutl/4))'; %Q3 1971 to Q2 2009
T=length(xtime);

[~,~,~,p]=size(tvKap);
[~,N,G]=size(tvLamb);
Horpl1=Hor+1;

%==== Cumulate?
WW_ObsFac=eye(4); 
Cumul_ObsFac=[1; 1; 0; 0];
Cumul=(codes==2|codes==5|codes==6);
%Switch Cumul off for now
Cumul_ObsFac=0*Cumul_ObsFac;
Cumul=0*Cumul;

%=== Select the interesting variables
nvSel = 1:9;

%=== Compute  factor IRFs and factor FEVDs (both wrt FCI shock) time-varying
FCIShockVec=zeros(G,1); FCIShockVec(FCIPos)=1;

tvIRFFChol=NaN(G,Horpl1,T); %empty container
tvIRFFChol0=NaN(G,Horpl1,T); % normed by FCI shock
tvFEVDFChol=NaN(G,Hor,T);
tvFCIResp0=NaN(T,1);
for tt=p+1:T;
    invP=inv(squeeze(tvP(tt,:,:)));
    StdevInnov=InnovVola(tt,:)';
    ATilde=invP*diag(StdevInnov);
    KapTildeComp=squeeze(tvKapTildeComp(tt,:,:));
    tvIRFFCholLoc=WL_VARIRFCOMP(KapTildeComp,ATilde,FCIShockVec,Hor);
    tvIRFFCholLoc=squeeze(tvIRFFCholLoc(:,1,:));
        
    ATildeExt=zeros(p*G,G); ATildeExt(1:G,1:G)=ATilde;
    S = warning('off','all'); % discard all div by zero warning messages
    [tvFEVDFCholLoc,FEV]=WL_VAR_FEVD(KapTildeComp,ATildeExt,G,Hor,0);
    S = warning('on','all');    
    tvFEVDFChol(:,:,tt)=squeeze(tvFEVDFCholLoc(:,FCIPos,:));
    
    tvIRFFChol(:,:,tt)=tvIRFFCholLoc;
    tvFCIResp0(tt)=ATilde(FCIPos,FCIPos);
    tvIRFFChol0(:,:,tt)=tvIRFFCholLoc/ATilde(FCIPos,FCIPos);
end;

FEVDObs = tvFEVDFChol(1:FCIPos,:,:);  

%=======================================================================
%==== Compute the tv FEVDs  
tvFEVDObsChol=NaN(nbCtr,Hor,T);
for tt=p+1:T;
    invP=inv(squeeze(tvP(tt,:,:)));
    StdevInnov=InnovVola(tt,:)';
    ATilde=invP*diag(StdevInnov);
    KapTildeComp=squeeze(tvKapTildeComp(tt,:,:));
    
    LambdaLoct=NaN(nbCtr,G);
    RvecLoct=NaN(nbCtr,1);
    rhovecLoct=NaN(nbCtr,1);
    for kk=1:nbCtr;
        nvSelkk=nvSel(kk);
        if isnan(nvSelkk);
            LambdaLoct(kk,:)=NaN;
            RvecLoct(kk)=NaN;
            rhovecLoct(kk)=NaN;
        else
            LambdaLoct(kk,:)=squeeze(tvLamb(tt,nvSelkk,:));
            RvecLoct(kk)=RvecEstFortv(nvSelkk);
            rhovecLoct(kk)=rhovecEstFortv(nvSelkk);
        end;
    end;
    FEVD=WL_FAVAR_FEVD(KapTildeComp,ATilde,LambdaLoct,RvecLoct,rhovecLoct,Cumul,Hor);
    % this result is N x G x Hor
    tvFEVDObsChol(:,:,tt)=squeeze(FEVD(:,FCIPos,:));
end;
FEVDCtr = tvFEVDObsChol;


cd ..\Results_IRs;
save(['FEVD_',IdentStrg], 'FEVDObs','FEVDCtr','xtime' );
cd ..\CodeData;

toc;
disp('...done');