%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino
% This program generates the FEVD
%==========================================================================

clear all;

%=======================================================================
%=== Load the result files:
tic;
cd ..\Results;
disp('Load: factors and Y, time-varying estimation results');

%=== Work with Baseline specification
% load fMatyMat_Baseline_FCI_Watson_nbfac10_G9 
% load Res_G_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%=== Work with Baseline FCI specification and GARCH
load fMatyMat_GARCH_FCI_Watson_nbfac10_G9 
load Res_G_GARCH_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_GARCH
load Res_N_GARCH_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_GARCH_AR1idio

%=== Work with specification where FCI is purged from US macro variables
% load fMatyMat_Baseline_FCI_Watson_purged_nbfac10_G9
% load Res_G_Baseline_FCI_Watson_purged_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FCI_Watson_purged_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%=== Work with specification with International FCI
% load fMatyMat_Baseline_FCI_International_nbfac10_G9
% load Res_G_Baseline_FCI_International_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FCI_International_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%=== Work with FSI specification
% load fMatyMat_Baseline_FSI_nbfac10_G9
% load Res_G_Baseline_FSI_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
% load Res_N_Baseline_FSI_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

cd ..\CodeData;
%=======================================================================

%=== SET these parameters
FCIPos = pos_fci; % Position of FCI
Hor=20;
cutl=0;
Hor=20;
cutl=0;

%[nbCtr,nbVar]=size(DispVar_nv); %
[nbCtr,nbVar]= size(variable_pos);
nbVar = nbVar-1; % disregard last column (FCI, for which we have separate IRF)

%=== Identify the individual result arrays/matrices:tvKap=Results_tv_VAR.RestrSmo.tvKap;
tvKap=Results_tv_VAR.RestrSmo.tvKap;
tvP=Results_tv_VAR.RestrSmo.tvP;
tvKapTildeComp=Results_tv_VAR.RestrSmo.tvKapTildeComp;
tvKapTildeCompMaxEig=Results_tv_VAR.RestrSmo.tvKapTildeCompMaxEig;
InnovVola=Results_tv_VAR.InnovVola;

tvLamb=Results_tv_Loadings.tvLambSmo;
lamb0Fortv=Results_tv_Loadings.lamb0Fortv;
RvecEstFortv=Results_tv_Loadings.RvecEstFortv;
rhovecEstFortv=Results_tv_Loadings.rhovecEstFortv;



xtime=((str2double(opt_starttime)+2/4):(1/4):(2012+3/4-cutl/4))'; %Q3 1971 to Q2 2009
T=length(xtime);

[~,~,~,p]=size(tvKap);
[~,N,G]=size(tvLamb);
Horpl1=Hor+1;

Horpl1=Hor+1;

%=== Shock vector
FCIShockVec=zeros(G,1); FCIShockVec(FCIPos)=1;

%==== Cumulate?
WW_ObsFac=ones(4,4);%diagWW(end-4:end);
Cumul_ObsFac=[1; 1; 0; 0];
Cumul=(codes==2|codes==5|codes==6);
%Switch Cumul off for now
Cumul_ObsFac=0*Cumul_ObsFac;
Cumul=0*Cumul;

%load the nv

%==== Compute  factor IRFs time-varying
tvIRFFChol=NaN(G,Horpl1,T); %empty container
tvIRFFChol0=NaN(G,Horpl1,T); % normed by FCI shock
tvFEVDFChol=NaN(G,Hor,T);
tvFCIResp0=NaN(T,1);

for tt = p+1:T;
    invP=inv(squeeze(tvP(tt,:,:)));
    StdevInnov=InnovVola(tt,:)';
    ATilde=invP*diag(StdevInnov);
    KapTildeComp=squeeze(tvKapTildeComp(tt,:,:));
    tvIRFFCholLoc=WL_VARIRFCOMP(KapTildeComp,ATilde,FCIShockVec,Hor);
    tvIRFFCholLoc=squeeze(tvIRFFCholLoc(:,1,:));
    tvIRFFChol(:,:,tt)=tvIRFFCholLoc;
    tvFCIResp0(tt)=ATilde(FCIPos,FCIPos);
    tvIRFFChol0(:,:,tt)=tvIRFFCholLoc/ATilde(FCIPos,FCIPos);
end;

%====  Compute  IRFs of obs factors time-varying
tvIRFObsFacChol=NaN(nbobs,Horpl1,T);

for tt=p+1:T;
    if  wantblockexogobs == 0
        LambdaLoct=zeros(nbobs,G);
    else
        LambdaLoct = [eye(nbobs),zeros(nbobs,G-nbobs)];
    end
    
    LambdaLoct(FCIPos)=1;
    tvIRFObsFacCholLoct=LambdaLoct*squeeze(tvIRFFChol0(:,:,tt));
    % ... and  rescale to get size of orig variable for a one unit shock of
    % the FCI in orig measuresements
    for ii=1:nbobs;
        tvIRFObsFacCholLoct(ii,:)=tvIRFObsFacCholLoct(ii,:)*WW_ObsFac(ii)/WW_ObsFac(FCIPos);
    end;
    for ii=1:nbobs;
        if Cumul_ObsFac(ii)==1;
            tvIRFObsFacCholLoct(ii,:)=1*cumsum(tvIRFObsFacCholLoct(ii,:));
            %tvIRFObsCholLoct(ii,:)=100*(tvIRFObsCholLoct(ii,:));
        end;
    end;
    tvIRFObsFacChol(:,:,tt)=tvIRFObsFacCholLoct;
end

%_____________________________________________
% Compute  IRFs of ALL obs variables time-varying
%_____________________________________________
tvIRFObsChol=NaN(N,Horpl1,T);

for tt=p+1:T;
    LambdaLoct=squeeze(tvLamb(tt,:,:));
    tvIRFObsCholLoct=LambdaLoct*squeeze(tvIRFFChol0(:,:,tt));
    % ... and  rescale to get size of orig variable for a one unit shock of
    % the FCI in orig measuresements
    for ii=1:N;
        tvIRFObsCholLoct(ii,:)=tvIRFObsCholLoct(ii,:)*diagWW(ii)/WW_ObsFac(FCIPos);
    end;
    for ii=1:N;
        if Cumul(ii)==1;
            tvIRFObsCholLoct(ii,:)=cumsum(tvIRFObsCholLoct(ii,:));
            %tvIRFObsCholLoct(ii,:)=100*(tvIRFObsCholLoct(ii,:));
        end;
    end;
    tvIRFObsChol(:,:,tt)=tvIRFObsCholLoct;
end;

%=== Extract the obs IRF for the selected variables by country and variables
tvIRFObsCholSel=NaN(nbCtr,nbVar,Horpl1,T);
for ii=1:nbCtr;
    for jj=1:nbVar;
        % check if the nv is there
        if not (variable_pos(ii,jj))== 0;
            idx  = variable_pos(ii,jj);
            tvIRFObsCholSel(ii,jj,:,:)=squeeze(tvIRFObsChol(idx,:,:));
            % Place the IRF by countries (1-9, rows) -->  disp(countries(ii,:))
            % and variables (1-24, columns) --> disp(variable_names{jj})
        end;
    end;
end;


%=== Smoothed shocks of FCI
ShocksFCI=NaN(T,1);
for tt=p+1:T;
    PLoc=squeeze(tvP(tt,:,:));
    KapArrLoc=squeeze(tvKap(tt,:,:,:));
    % Res is the vector VolaMat * eps_t
    Res=PLoc*fMat(tt,:)';
    for pp=1:p;
        KapLoc=squeeze(KapArrLoc(:,:,pp));
        Res=Res-KapLoc*fMat(tt-pp,:)';
    end;
    ShocksFCI(tt)=Res(FCIPos);
    %ShocksFCI(tt)=Res(FCIPos)*(1/InnovVola(tt,FCIPos));
end;

%=== Collect for saving and save
IRFCtrVar=tvIRFObsCholSel;
IRFObsFac=tvIRFObsFacChol;
FCIVola=InnovVola(:,FCIPos);

cd ..\Results_IRs;
save(['IRF_',IdentStrg], 'IRFCtrVar', 'IRFObsFac','FCIVola','ShocksFCI','xtime' );
cd ..\CodeData;
