%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino
% This program estimates IRF with CI for selected variables
% Replication file for Tables 2 to 4
%==========================================================================


clear all;
close all;
clc;


countries = ['US '; 'CA '; 'UK '; 'FR '; 'DE '; 'IT '; 'ES '; 'JP ';'AU '];
Nc = length(countries); % G9 countries 

cd ..\Results;
load fMatyMat_Baseline_nbfac10_G9  

cd ..\Results;
load fMatyMat_Baseline_FCI_Watson_nbfac10_G9 
load Res_G_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
load Res_N_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
%=======================================================================


cd ..\Results_IRs;  % or whatever file you want to go
load FEVD_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
load IRF_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
%==========================================================================

cd ..\CodeData;


FCIpos = 1; % FCI is ordered before the unobservable factors
Obspos = 1; % there is only 1 observable factor (FCI)


%===  Identify the individual result arrays/matrices:
tvKap=Results_tv_VAR.RestrSmo.tvKap;
tvP=Results_tv_VAR.RestrSmo.tvP;
tvKapTildeComp=Results_tv_VAR.RestrSmo.tvKapTildeComp;
tvKapTildeCompMaxEig=Results_tv_VAR.RestrSmo.tvKapTildeCompMaxEig;
InnovVolaFortv=Results_tv_VAR.InnovVola;
VAREStates=Results_tv_VAR.States.asmosRestr;    %NEWCI: the results of the tv VAR estimation now also contain the smoothed state series (from which the tv Kap, tv P are derived)
VARVariStates=Results_tv_VAR.States.PsmosRestr; %NEWCI: the results of the tv VAR estimation now also contain the Var-Cov matrices of the smoothed state series

tvLamb=Results_tv_Loadings.tvLambSmo;
tvLambVari=Results_tv_Loadings.tvLambSmoVari;   %NEWCI: for the tv Lambda. also the corresponding set of Var-Cov matrices is stored and loaded here
lamb0Fortv=Results_tv_Loadings.lamb0Fortv;
RvecEstFortv=Results_tv_Loadings.RvecEstFortv;
rhovecEstFortv=Results_tv_Loadings.rhovecEstFortv;


%=== Set some parameters
Hor             = 5;    % Horizon over which to compute IRFs
p               = size(tvKap, 4); % Lag length
[T,N,G]         = size(tvLamb);
seltimepoints   = p+1:T; % time over which to compute Confidence Bands                         
nbseltimepoints = length(seltimepoints);
M               = 1000;        % target number of successful draws (explosive draws are discarded)
myquantiles     = [.05; .95];  % desired probs for confibands
cumulh          = 0; % do not cumulate for now (will do it later)

%=== Initialize Shock Vector: 1-sized shocks
ShockVec=zeros(G,1); 
ShockVec(FCIpos)=1;

%=== Reminder of where are the variable
    %                    1       2       3           4            5              6         7       8    9        10       11          12
%variable_names = {{'gdp'},{'cons'},{'inv'},{'residinv'},{'nonresidinv'},{'govcons'},{'ex'},{'im'},{'ip'},{'unempl'},{'gdpdefl'},{'cpi'},...
 %   {'exp'},{'imp'},{'equityp'},{'housep'},{'credit'},{'stint'},{'ltint'},{'reer'},{'erusd'},{'govd'},{'tfp'},{'observ'}};
    %  13      14       15          16          17      18          19       20      21          22      23      24 

%=== Select the variables you want to compute CI for
%var_display = [11 18 15]; % GDP deflator, R and Equity prices (Table 3 in the paper)
var_display = [7,8,20]; % GDP deflator, R and Equity prices (Table 3 in the paper)

% var_display = [1:3 11 18 15 7 8 20]; % Full set of variables reported in the paper


pointIRF      = zeros(Hor+1, size(var_display,2), 9, nbseltimepoints);
quantilesIRF  = zeros(Hor+1, 2, size(var_display,2), 9, nbseltimepoints);
pointIRFFac = []; quantilesIRFFac = [];
for jj=1:nbseltimepoints;
    disp(['Time ', num2str(jj)])
    th=seltimepoints(jj);
    
    for counter = 1:size(var_display,2)
        vvar1 = var_display(counter);
        disp(['Getting CI for variable ', num2str(vvar1)])
        vvar2= variable_pos(:,vvar1);
        
        for counter2 = 1:size(vvar2,1)
            nvh = vvar2(counter2);
            if not (nvh==0)
                [temp1,~,temp2,info] = WL_tvFFAVAR_IRFwtBands(VAREStates,VARVariStates,tvLamb,tvLambVari,ShockVec,p,th,nvh,M,myquantiles,Hor,cumulh);
                pointIRF(:,counter,counter2,jj) = temp1;
                quantilesIRF(:,:,counter,counter2,jj) = temp2;
                clear temp1 temp2
                %
            end
        end
    end
    
    save tempCI_all
end;

%=== Multiply the IRFs by the original stdev of the variables, and express them in % points where suitable
pointIRF2     = zeros(Hor+1, size(var_display,2), 9, nbseltimepoints);
quantilesIRF2 = zeros(Hor+1, 2, size(var_display,2), 9, nbseltimepoints);

for counter = 1:size(var_display,2)
    vvar1 = var_display(counter);
    vvar2= variable_pos(:,vvar1);
    
    for counter2 = 1:size(vvar2,1)
        nvh = vvar2(counter2);
        if not (nvh==0)
            if not (vvar1 == 18)% R ias already in % points 
                pointIRF2(:,counter,counter2,:) = pointIRF(:,counter,counter2,:).*100*diagWW(nvh);
                quantilesIRF2(:,:,counter,counter2,:) = quantilesIRF(:,:,counter,counter2,:).*100*diagWW(nvh);
            else
                pointIRF2(:,counter,counter2,:) = pointIRF(:,counter,counter2,:).*diagWW(nvh);
                quantilesIRF2(:,:,counter,counter2,:) = quantilesIRF(:,:,counter,counter2,:).*diagWW(nvh);
            end
        end
    end
end

 
save ResultsCI_all

%===========================================================================
%=== Build Table
%=== Select HORIZONS over which to compute IRF table
hor = [1,5]; % default is impact and one-year ahead

%=== Select PERIODS over which to compute IRF table
% Start in 2007Q3 (145 Recession starts - NBER) and Stop in 2008Q2 (148): Pre Lehman bankruptcy filing (15.09.2008 - Q3)
% 2008Q3 - 2009Q2 (Recession ends - NBER) (149-152)
% 2008Q1 - 2009Q2 (Recession ends - NBER) (147-152)
% Post-Lehman period: 149 (2008Q3) - 166 (2012Q4)
PERIODS = {{1,14:47,54:61,144:145},{62:64,82:117,122:143},{2:13,48:53,65,66:81,118:121},{146:151},{146:165}}; % If sample starts in 1971

%=== Make sure that:
%1) pointIRF2     is  hor x country x time
%1) quantilesIRF2 is  hor x quantile x country x time

IRF = []; % this will contain the table, computed progressively over each variable

for mtemp = 1:size(var_display,2) % run over selected variables
    mm = var_display(mtemp);
    point  = squeeze(pointIRF2(:,mtemp,:,:)); % hor x country x time
    quant  = squeeze(quantilesIRF2(:,:,mtemp,:,:));
    
    if not (mm == [18,20]) % do not cumulate R and REER
        point  = cumsum(point,1); % Cumulate relevant IRF over horizons
        quant  = cumsum(quant,1); % Cumulate relevant IRF over horizons
    end
    
    IRFtemp = zeros(9,5,2,2); % country x period x hor x info
    counter = 0; % counter
    for h=[1,5]
        counter = counter +1;
        for i=1:9
            for j = 1:size(PERIODS,2);
                per = cell2mat(PERIODS{j}); % Select periods over which to compute IRF
                
                tempquant  = squeeze(quant(h,:,i,per));
                Ttemp      = size(tempquant,2);
                mask       = ones(Ttemp,1); % 1 if the IRF is statistically significant
                for tt = 1:Ttemp
                    if tempquant(1,tt)<0 && tempquant(2,tt)>0
                        mask(tt)=0; % the irf is not statistically significant
                    end
                end
                IRFtemp(i,j,counter,1) = mean(squeeze(point(h,i,per)));
                IRFtemp(i,j,counter,2) = round(mean(mask));
            end
        end
    end
    
    IRFtemp2 = [squeeze(IRFtemp(:,:,1,1)) squeeze(IRFtemp(:,:,2,1)); squeeze(IRFtemp(:,:,1,2)), squeeze(IRFtemp(:,:,2,2))];
    IRF = [IRF; IRFtemp2];
    clear IRFtemp2 IRFtemp point quant
end