%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino

%==== This codes plots the IRF and FEVD of GDP, across horizons and years
% It reproduces all Figures 1in the paper

%=== Specifications allowed
% US block is ENDOGENOUS
% Volatility modeled as BASELINE or as GARCH
%==========================================================================
clear all;
close all;
clc;

countries = ['US '; 'CA '; 'UK '; 'FR '; 'DE '; 'IT '; 'ES '; 'JP ';'AU '];
Nc = length(countries); % G9 countries

cd ..\Results_IRs;

load FEVD_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
load IRF_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio

%==========================================================================
cd ..\CodeData;

variables = [1, 2, 3, 7, 8, 11, 13, 14, 15, 16, 17, 18, 19, 20, 23, 6, 10, 22, 4, 21];


selyears =  [1976; 1986; 1996; 2008; 2011 ]; %; 2010
map = jet(length(selyears));
indexxtime = [[1:length(xtime)]' xtime];
maxhor = 20; % IRF horizon

fci_pos = 1;

observ  = xlsread('dataint_2015','FCI','e9:e178'); %17
%==========================================================================
%=== Figure 1 to 3 in the paper
figure(1)
% Figure 1: US Financial condition index
plot(xtime,observ(5:end),'k','LineWidth',2), axis tight
set(gcf, 'PaperPosition', [0 0 10 5]); %Position plot at left hand corner with width 5 and height 5.
set(gcf, 'PaperSize', [10 5]); %Set the paper to have width 5 and height 5.
 
figure(2)
% Figure 2: Time-varying FCI volatility
plot(xtime,FCIVola,'k','LineWidth',2), axis tight
set(gcf, 'PaperPosition', [0 0 10 5]); %Position plot at left hand corner with width 5 and height 5.
set(gcf, 'PaperSize', [10 5]); %Set the paper to have width 5 and height 5.

figure(3)
% Figure 3: IRfs of FCI over time
mesh(xtime,0:maxhor, squeeze(IRFObsFac(fci_pos,1:maxhor+1,:))); axis tight;...
set(gcf, 'PaperPosition', [0 0 10 5]); %Position plot at left hand corner with width 5 and height 5.
set(gcf, 'PaperSize', [10 5]); %Set the paper to have width 5 and height 5.

%=== Plot IRF and FEVD for FCI series (not shown in the paper)
hor     = [1,5,9];
fci_pos = 1; % position of FCI in observables

figure(22)
% FCI IRFs to FCI shock (selected horizons)
subplot(1,3,1), plot(xtime,squeeze(IRFObsFac(fci_pos,hor,:)),'LineWidth',2), axis tight;...
    hold on, plot(xtime,zeros(length(xtime),1),'k'); title('IRFs (sel. horizons)'); legend('0q','4q','8q'),
% FCI IRFs to FCI shock (sel. points in time)
subplot(1,3,2),
for j = 1:length(selyears)
    hold on, plot(0:maxhor,squeeze(IRFObsFac(fci_pos,1:maxhor+1,sum((indexxtime(:,2)==selyears(j,:)).*indexxtime(:,1)))),'LineWidth',2,'Color',map(j,:))
end
hold on, plot(0:maxhor,zeros(maxhor+1,1),'k'); legend(num2str(selyears)); title('IRFs (sel. points in time)')
% FEVD of FCI
subplot(1,3,3), plot(xtime,squeeze(FEVDObs(fci_pos,[4 12],:)*100),'LineWidth',2); ...
    hold on, plot(xtime,zeros(length(xtime),1),'k'); axis tight; title('FEV shares expl. by FCI shock'); legend('4q','20q','Location','SE')

%==========================================================================
gdp_pos = 1;
%=== Plot FEVD of GDP - Figure 6 in the paper
% uncomment if you want to look at point IRFs
figure(6)
hor2    = [1,5,9];
for i = 1:Nc,
%     subplot(3,Nc,i),
%     hold on, plot(xtime,squeeze(IRFCtrVar(i,gdp_pos,hor2,:)*100),'LineWidth',2)
%     hold on, plot(xtime,zeros(length(xtime),1),'k')
%     axis([xtime(1) xtime(end) nanmin(nanmin(nanmin(IRFCtrVar(:,gdp_pos,hor2,:)*100))) nanmax(nanmax(nanmax(IRFCtrVar(:,gdp_pos,hor2,:)*100)))]),
%     title(countries(i,:)),
%     if i == Nc, legend('0q','4q','8q','Orientation','vertical'), end
%     
%     %=== Plot IRF of GDP  at selected points in time
%     subplot(3,Nc,Nc+i),
%     for j = 1:length(selyears)
%         hold on, plot(0:maxhor,squeeze(IRFCtrVar(i,gdp_pos,1:maxhor+1,sum((indexxtime(:,2)==selyears(j,:)).*indexxtime(:,1)))*100),'LineWidth',2,'Color',map(j,:))
%     end
%     hold on, plot(0:maxhor,zeros(maxhor+1,1),'k'); title(countries(i,:))
%     axis tight
%     if i == 1, legend(num2str(selyears),'Orientation','horizontal'), ylabel('IRFs (sel. points in time)'), end
    
    % FEVD
    subplot(1,Nc,i),% subplot(3,Nc,2*Nc+i),
    hold on, plot(xtime,squeeze(FEVDCtr(i,4,:)*100),'b','LineWidth',2)
    hold on, plot(xtime,squeeze(FEVDCtr(i,maxhor,:)*100),'r--','LineWidth',2)
    axis([xtime(1) xtime(end) nanmin(nanmin(nanmin(FEVDCtr(:,[4,maxhor],:)*100))) nanmax(nanmax(nanmax(FEVDCtr(:,[4,maxhor],:)*100)))]),
    if i == 1, legend('4q','20q','Orientation','horizontal'), ylabel('FEV expl. by FCI shock'), end
    title(countries(i,:)),
    
end

%==========================================================================
%=== FIGUREs 4 and 5 in the paper
xtimebackup = xtime;
load('BASELINE_Results_CI_GDP', 'quantilesIRF2','xtime','pointIRF2');

% Mesh plot of IRFs over time - Figure 4 in the paper
figure(4)
for i = 1:Nc,
    subplot(3,3,i),   mesh(xtime(2:end), 1:21, squeeze(pointIRF2(:,i,:))),  axis tight; title(countries(i,:)),
    
    
end
set(gcf, 'PaperPosition', [0 0 10 7]); %Position plot at left hand corner with width 5 and height 5.
set(gcf, 'PaperSize', [10 7]); %Set the paper to have width 5 and height 5.
%saveas(4,'PAPER\Figure2_BASELINE_MESH.pdf','pdf');
% [ax1,h1]=suplabel('Impulse responses (point estimates)', 't') ;
% set(h1,'FontSize',12, 'FontWeight','bold')

% Plot IRF of GDP by quarters with 90% confidence bands - Figure 5 in the paper
xtime2 = xtime(2:end);
figure(5)
for i = 1:Nc
    subplot(3,Nc,i), plot(xtime2,squeeze(quantilesIRF2(1,:,i,:)),'k:','LineWidth',2), hold on, plot(xtime2,squeeze(pointIRF2(1,i,:)),'r','LineWidth',2), hold on, plot(xtime2,zeros(length(xtime2),1),'k'), axis([min(xtime2) max(xtime2) min(min(min(squeeze(quantilesIRF2(2,:,:,:))))) max(max(max(squeeze(quantilesIRF2(2,:,:,:)))))]);...
        title(countries(i,:)); if i == 1, ylabel('0q'), end
    subplot(3,Nc,i+Nc), plot(xtime2,squeeze(quantilesIRF2(5,:,i,:)),'k:','LineWidth',2), hold on, plot(xtime2,squeeze(pointIRF2(5,i,:)),'r','LineWidth',2), hold on, plot(xtime2,length(xtime2),'k'), axis([min(xtime2) max(xtime2) min(min(min(squeeze(quantilesIRF2(5,:,:,:))))) max(max(max(squeeze(quantilesIRF2(5,:,:,:)))))]), if i == 1, ylabel('4q'), end
    subplot(3,Nc,i+Nc*2), plot(xtime2,squeeze(quantilesIRF2(9,:,i,:)),'k:','LineWidth',2), hold on, plot(xtime2,squeeze(pointIRF2(9,i,:)),'r','LineWidth',2), hold on, plot(xtime2,length(xtime2),'k'), axis([min(xtime2) max(xtime2) min(min(min(squeeze(quantilesIRF2(9,:,:,:))))) max(max(max(squeeze(quantilesIRF2(9,:,:,:)))))]), if i == 1, ylabel('8q'), end
    
    %hgexport(gcf, [pathname,cell2mat(variable_names{j}),'.jpeg'], hgexport('factorystyle'), 'Format', 'jpeg');
end
set(gcf, 'PaperPosition', [0 0 13 7]); %Position plot at left hand corner with width 5 and height 5.
set(gcf, 'PaperSize', [13 7]); %Set the paper to have width 5 and height 5.
%saveas(5,'PAPER\Figure2_BASELINE_CI.pdf','pdf');


%==========================================================================

% PLOT FCIs TOGETHER

% List:
% FCI Hatzius et al. -updated dataset	(e)
% FCI - International measure	(f)
% FSI* Hubrich and Tetlow	(g)
% FCI Hatzius et al. purged (h)


xtime = xtimebackup;

fci = xlsread('dataint_2015','FCI','e9:e182'); %17
fcint  = xlsread('dataint_2015','FCI','f9:f182'); %17
fciss  = xlsread('dataint_2015','FCI','g9:g182'); %17
fci_purge = xlsread('dataint_2015','FCI','h9:h182'); %17
figure(7)
plot(xtime, fci(9:end), 'k','LineWidth',2); hold on; plot(xtime, fcint(9:end), 'b-.','LineWidth',2); hold on; plot(xtime, fciss(9:end), 'r--','LineWidth',2);...
    hold on; plot(xtime, fci_purge(9:end), 'g--+','LineWidth',2); axis tight; legend('Baseline FCI', 'International FCI', 'FSI', 'Baseline FCI - US macro aggregates removed','Location', 'SW')
set(gcf, 'PaperPosition', [0 0 10 5]); %Position plot at left hand corner with width 5 and height 5.
set(gcf, 'PaperSize', [10 5]); %Set the paper to have width 5 and height 5.
%saveas(7,'FCI_comparison.pdf','pdf');


%=== To replicate below, you first have to re-estimate the model after
%selecting the desired FCI specification (opt_fci) in AELM_prepareData.m

% cd ../Results_IRs
% load('IRF_Baseline_FCI_International_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio','FCIVola');
% vola_FCINT = FCIVola; clear FCIVola
% load('IRF_Baseline_FCI_Watson_purged_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio','FCIVola');
% vola_FCIWp = FCIVola; clear FCIVola
% load('IRF_Baseline_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev','FCIVola');
% vola_FCIW  = FCIVola; clear FCIVola
% load('IRF_Baseline_FSI_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio','FCIVola');
% vola_FSI   = FCIVola; clear FCIVola
% 
% figure(8)
% plot(xtime, vola_FCIW, 'k','LineWidth',2); hold on; plot(xtime, vola_FCINT, 'b-.','LineWidth',2);  hold on; plot(xtime(17:end), vola_FSI, 'r--','LineWidth',2);...
%     hold on; plot(xtime, vola_FCIWp, 'g--+','LineWidth',2); axis tight; legend('Baseline FCI', 'International FCI', 'FSI', 'Baseline FCI - US macro aggregates removed','Location', 'SW')
% set(gcf, 'PaperPosition', [0 0 10 5]); %Position plot at left hand corner with width 5 and height 5.
% set(gcf, 'PaperSize', [10 5]); %Set the paper to have width 5 and height 5.
% %saveas(8,'FCIvola_comparison.pdf','pdf');

