%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino
% This program estimates IRF with CI for selected variables
% Replication file for Figures 4 and 5
%==========================================================================


clear all;
close all;


countries = ['US '; 'CA '; 'UK '; 'FR '; 'DE '; 'IT '; 'ES '; 'JP ';'AU '];
Nc = length(countries); % G9 countries 

 
cd ..\Results;

load('fMatyMat_Baseline_FCI_Watson_nbfac10_G9','diagWW')
load Res_G_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev
load Res_N_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
%=======================================================================


cd ..\Results_IRs;  % or whatever file you want to go
load FEVD_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
load IRF_Baseline_FCI_Watson_nbfac10_G9_ObsEndog_p1_VolaObs_AffLev_AR1idio
%==========================================================================

cd ..\CodeData;


FCIpos = 1; % FCI is ordered before the unobservable factors
Obspos = 1; % there is only 1 observable factor (FCI)


%===  Identify the individual result arrays/matrices:
tvKap=Results_tv_VAR.RestrSmo.tvKap;
tvP=Results_tv_VAR.RestrSmo.tvP;
tvKapTildeComp=Results_tv_VAR.RestrSmo.tvKapTildeComp;
tvKapTildeCompMaxEig=Results_tv_VAR.RestrSmo.tvKapTildeCompMaxEig;
InnovVolaFortv=Results_tv_VAR.InnovVola;
VAREStates=Results_tv_VAR.States.asmosRestr;    %NEWCI: the results of the tv VAR estimation now also contain the smoothed state series (from which the tv Kap, tv P are derived)
VARVariStates=Results_tv_VAR.States.PsmosRestr; %NEWCI: the results of the tv VAR estimation now also contain the Var-Cov matrices of the smoothed state series

tvLamb=Results_tv_Loadings.tvLambSmo;
tvLambVari=Results_tv_Loadings.tvLambSmoVari;   %NEWCI: for the tv Lambda. also the corresponding set of Var-Cov matrices is stored and loaded here
lamb0Fortv=Results_tv_Loadings.lamb0Fortv;
RvecEstFortv=Results_tv_Loadings.RvecEstFortv;
rhovecEstFortv=Results_tv_Loadings.rhovecEstFortv;


%=== Set some parameters
Hor             = 20;    % Horizon over which to compute IRFs
p               = size(tvKap, 4); % Lag length
[T,N,G]         = size(tvLamb);
seltimepoints   = p+1:T; % time over which to compute Confidence Bands                         
nbseltimepoints = length(seltimepoints);
M               = 1000;        % target number of successful draws (explosive draws are discarded)
myquantiles     = [.05; .95];  % desired probs for confibands
cumulh          = 0; % do not cumulate for now (will do it later)

%=== Initialize Shock Vector: 1-sized shocks
ShockVec=zeros(G,1); 
ShockVec(FCIpos)=1;

%=== Reminder of where the variables are
    %                    1       2       3           4            5              6         7       8    9        10       11          12
%variable_names = {{'gdp'},{'cons'},{'inv'},{'residinv'},{'nonresidinv'},{'govcons'},{'ex'},{'im'},{'ip'},{'unempl'},{'gdpdefl'},{'cpi'},...
 %   {'exp'},{'imp'},{'equityp'},{'housep'},{'credit'},{'stint'},{'ltint'},{'reer'},{'erusd'},{'govd'},{'tfp'},{'observ'}};
    %  13      14       15          16          17      18          19       20      21          22      23      24 

%=== Select the variables you want to compute CI for
var_display = 1; % GDP

pointIRF      = zeros(Hor+1, 9, nbseltimepoints);
quantilesIRF  = zeros(Hor+1, 2, 9, nbseltimepoints);
pointIRFFac = []; quantilesIRFFac = [];
for jj=1:nbseltimepoints;
    disp(['Time ', num2str(jj)])
    th=seltimepoints(jj);
    
    vvar2= 1:9; % the first 9 variables are the GDP series
    for counter2 = 1:size(vvar2,2)
        nvh = vvar2(counter2); % get the variable we are interested in
        
        [temp1,~,temp2,info] = WL_tvFFAVAR_IRFwtBands(VAREStates,VARVariStates,tvLamb,tvLambVari,ShockVec,p,th,nvh,M,myquantiles,Hor,cumulh);
        pointIRF(:,counter2,jj) = temp1;
        quantilesIRF(:,:,counter2,jj) = temp2;
        clear temp1 temp2
        
    end
    
    
    save tempCI_GDP
end;

pointIRF2     = zeros(Hor+1, 9, nbseltimepoints);
quantilesIRF2 = zeros(Hor+1, 2, 9, nbseltimepoints);
for i = 1:9
     pointIRF2(:,i,:) = pointIRF(:,i,:)*100*diagWW(i);
     quantilesIRF2(:,:,i,:) = quantilesIRF(:,:,i,:)*100*diagWW(i);        
end

save BASELINE_Results_CI_GDP
 
%=========================================================================
%=== Plot IRFs
% Mesh plot of IRFs over time - Figure 4 in the paper
nbcountr = 9;
xtime = xtime(2:end);
figure(4)
for i = 1:nbcountr
    subplot(3,3,i), mesh(squeeze(pointIRF2(:,i,:))), axis tight;
end
 
% Plot IRF of GDP by quarters with 90% confidence bands - Figure 5 in the paper
figure(5)
for i = 1:Nc
    subplot(3,Nc,i), plot(xtime,squeeze(quantilesIRF2(1,:,i,:)),'k:','LineWidth',2), hold on, plot(xtime,squeeze(pointIRF2(1,i,:)),'r','LineWidth',2), hold on, plot(xtime,zeros(T-p,1),'k'), axis([min(xtime) max(xtime) min(min(min(squeeze(quantilesIRF2(2,:,:,:))))) max(max(max(squeeze(quantilesIRF2(2,:,:,:)))))]);...
        title(countries(i,:)); if i == 1, ylabel('0q'), end
    subplot(3,Nc,i+Nc), plot(xtime,squeeze(quantilesIRF2(5,:,i,:)),'k:','LineWidth',2), hold on, plot(xtime,squeeze(pointIRF2(5,i,:)),'r','LineWidth',2), hold on, plot(xtime,zeros(T-p,1),'k'), axis([min(xtime) max(xtime) min(min(min(squeeze(quantilesIRF2(5,:,:,:))))) max(max(max(squeeze(quantilesIRF2(5,:,:,:)))))]), if i == 1, ylabel('4q'), end
    subplot(3,Nc,i+Nc*2), plot(xtime,squeeze(quantilesIRF2(9,:,i,:)),'k:','LineWidth',2), hold on, plot(xtime,squeeze(pointIRF2(9,i,:)),'r','LineWidth',2), hold on, plot(xtime,zeros(T-p,1),'k'), axis([min(xtime) max(xtime) min(min(min(squeeze(quantilesIRF2(9,:,:,:))))) max(max(max(squeeze(quantilesIRF2(9,:,:,:)))))]), if i == 1, ylabel('8q'), end
    [ax1,h1]=suplabel('GDP', 't');
    set(h1,'FontSize',15)
    %hgexport(gcf, [pathname,cell2mat(variable_names{j}),'.jpeg'], hgexport('factorystyle'), 'Format', 'jpeg');
end
set(gcf, 'PaperUnits', 'inches');
x_width=17.5 ;y_width=10.325;
set(5, 'PaperPosition', [0 0 x_width y_width]); %
 

%=========================================================================
%=== DISP TABLE OF IRFs FOR GDP to a 1-unit FCI shock
%=== Select HORIZONS over which to compute IRF table
hor = [1,5]; % default is impact and one-year ahead

%=== Select PERIODS over which to compute IRF table
% Start in 2007Q3 (145 Recession starts - NBER) and Stop in 2008Q2 (148): Pre Lehman bankruptcy filing (15.09.2008 - Q3)
% 2008Q3 - 2009Q2 (Recession ends - NBER) (149-152)
% 2008Q1 - 2009Q2 (Recession ends - NBER) (147-152)
% Post-Lehman period: 149 (2008Q3) - 166 (2012Q4)
PERIODS = {{1,14:47,54:61,144:145},{62:64,82:117,122:143},{2:13,48:53,65,66:81,118:121},{146:151},{146:165}}; % If sample starts in 1971

%=== Make sure that:
%1) pointIRF2     is  hor x country x time
%1) quantilesIRF2 is  hor x quantile x country x time

point  = pointIRF2; % hor x country x time
point  = cumsum(point,1); % Cumulate relevant IRF over horizons
quant  = quantilesIRF2;
quant  = cumsum(quant,1); % Cumulate relevant IRF over horizons

IRF = zeros(9,5,2,2); % country x period x hor x info
counter = 0; % counter
for h=[1,5]
    counter = counter +1;
    for i=1:9
        for j = 1:size(PERIODS,2);
            per = cell2mat(PERIODS{j}); % Select periods over which to compute IRF
            
            tempquant  = squeeze(quant(h,:,i,per));
            Ttemp      = size(tempquant,2);            
            mask       = ones(Ttemp,1);
            for tt = 1:Ttemp
                if tempquant(1,tt)<0 && tempquant(2,tt)>0
                    mask(tt)=0;
                end
            end
            IRF(i,j,counter,1) = mean(squeeze(point(h,i,per)));
            IRF(i,j,counter,2) = round(mean(mask));
        end
    end
end

IRFGDP = [squeeze(IRF(:,:,1,1)) squeeze(IRF(:,:,2,1)); squeeze(IRF(:,:,1,2)), squeeze(IRF(:,:,2,2))];

disp('%=========================================================================')
disp('Impact and one-year ahead impulse responses of GDP to a 1-unit FCI shock, over selected periods')
disp('The first block is 9 (# of countries) times 5*2 ( # of periods x horizons)')
disp('The second block reports in position (i,j) 1 if the average IRF in position (i-9,j) is significant')
disp(IRFGDP)