%==========================================================================
% The International Transmission of financial shocks (2015)
% Abbate, Eickmeier, Lemke, Marcellino
% This program loads data and saves data and factors.
%==========================================================================
clear all
clc
close all

%==========================================================================
%=== Set Estimation parameters
%==========================================================================
countries = ['US'; 'CA'; 'UK'; 'FR'; 'DE'; 'IT'; 'SP'; 'JP';'AU'];
Nc = length(countries); % G9 countries: US, Canada, UK, FR, DE, IT,Spain, JP, Australia
Nv = 24; % Number of variables

%=== Which FCI measure?
opt_fci = 'FCI_Watson_step1'; % FCI Hatzius et al. -updated dataset	(e)

%==========================================================================
%=== Select start and end dates of estimation sample
opt_endtime   = '2012'; %2009, 2010, 2012
opt_starttime = '1971'; % 1973
sheetname     = 'dataint_2015'; 
%===  How do you want to model volatility?
volatype = 2;     % functional form of vola.  !!!BASELINE: =  2, GARCH: = 5
vola_fun = 0;     % relevant iff volatype = 2 (specifies affine function)             
%=== Volatility options
% vola_fun = 0;  % 'Baseline: affine function of past observables';  
% vola_fun = 1 % 'Baseline: factors affine function of squared past observables';    
% vola_fun = 2 % 'Baseline: affine function of contemporaneous observables';    
% vola_fun = 3 % 'Baseline: affine function of squared latent factors and FCI^2';    
% vola_fun = 4 % 'Baseline: affine function and uncertainty index added to set of regressors';        

if  vola_fun == 0 && not(volatype == 5);
    opt_vola = 'affine function of past observables';
elseif  vola_fun == 1
    opt_vola = 'affine function of squared past observables';    
elseif  vola_fun == 2
    opt_vola = 'affine function of contemporaneous observables';    
elseif  vola_fun == 3
    opt_vola = 'affine function of squared latent factors and FCI^2';    
elseif  vola_fun == 4
    opt_vola = 'affine function and uncertainty index added to set of regressors';    
end
 
%=== Set Factor parameters
rplusM    = 10;       % Number of factors
optionrem = 2;        % 0/1/2: 0: remove observables from latent factors using the Boivin-Giannoni procedure
                      % 1: do not remove observables from latent factors
                      % 2: remove obs. from latent using orthog. complement
p         = 1;        % VAR process of the factors
normaliz  = 0;        % Normalization of factors/factor loadings 
wantblockexogobs = 0; %0: USA endog (baseline), 1: version where the observables among the factors are treated as
                      % blockexog  in the VAR. Also factor volas of the obs do not depend on the
                      % other factors,  but only on their own lags (robustness check)
 
%=== Display estimation features
disp('%=================================================================================')
disp(['Estimation ', 'from ' opt_starttime ' to ' opt_endtime])
disp(['FCI measured as ',  opt_fci])
disp(['Number of Factors = ' num2str(rplusM) ', following a VAR(' num2str(p) ') process'])
disp(['Factor volatility modeled as ', opt_vola])
if wantblockexogobs == 1;
    disp('FCI treated as exogenous')
end
disp('%==================================================================================')
 
%=== Name the results
if wantblockexogobs == 0
   IdentStrg=['Baseline_' ,'nbfac',num2str(rplusM),'_G9' ];
else
   IdentStrg=['INIT08Q2_Baseline_' , vola_fun ,'_','nbfac',num2str(rplusM),'_G9','_USEXOG']; 
end

%IdentStrg=['INIT08Q2_GARCH_MeanAdj', opt_fci ,'_nbfac',num2str(rplusM),'_G9','_FCIonly']; 
%IdentStrg=['INIT08Q4_', opt_fci ,'_nbfac',num2str(rplusM),'_G9','_FCIonly']; 
%==========================================================================%
%SHOW THE FOLLOWING VARIABLES
if wantblockexogobs == 0
    obs = [24 1];
else
     obs = [24 1; 24 2; 24 3; 24 4]; 
end
nbobs = size(obs,1); % number of observables among the factors
    
v1 = obs;  %US variables 

variables = [1, 2, 3, 7, 8, 11, 13, 14, 15, 16, 17, 18, 19, 20, 23, 6, 10, 22, 4, 21];

%variables = 1:2;%23;
v = v1;
vv = [];
for i = 1:length(variables)
    vv(:,:,i) = [variables(i)*ones(1,Nc)
        1:Nc]';
    v = [v; vv(:,:,i)];
end

 
%==========================================================================%
%=== Load DATA   
[Y,WW,MMM,nobs,nv,id,codes,varnames, Nsub0, pos_fci] = dataload(v, obs, opt_endtime, opt_starttime, sheetname, opt_fci,wantblockexogobs); 
OBS = Y(:,nobs)*WW(nobs,nobs); %observables

% pos_fci returns the position of the FCI among the observables
Y = Y(:,1:nobs(1)-1); %remove observable(s) from total dataset
[T,N] = size(Y);

idtmp = id(1:end-cols(OBS),:);
YF = Y;
%                    1       2       3           4            5              6         7       8    9        10       11          12
variable_names = {{'gdp'},{'cons'},{'inv'},{'residinv'},{'nonresidinv'},{'govcons'},{'ex'},{'im'},{'ip'},{'unempl'},{'gdpdefl'},{'cpi'},...
    {'exp'},{'imp'},{'equityp'},{'housep'},{'credit'},{'stint'},{'ltint'},{'reer'},{'erusd'},{'govd'},{'tfp'},{'observ'}};
    %  13      14       15          16          17      18          19       20      21          22      23      24
    
prova = zeros(Nc-wantblockexogobs,24);
prova(:,1) = 1:Nc-wantblockexogobs;
for i =2:24
        prova(:,i) = prova(:,i-1) + Nc-wantblockexogobs;    
end
% Dataset is unbalanced; % remove missing obs (useful later for IRFs)
prova(6-wantblockexogobs:7-wantblockexogobs,5) = [0;0]; % remove IT and SP Nonresidential Investment
prova(end,end-1) = 0; % remove AU TFP
prova(nbobs+1:end,end) = zeros(Nc-wantblockexogobs-nbobs,1); % only US FCI
prova(7-wantblockexogobs,18) = 0; % remove SP short-term R
if not(wantblockexogobs)
    prova(1,21) = 0; % remove US erusd
end
% Tells you which variable is which in the dataset
variable_pos = zeros(Nc-wantblockexogobs,24);
num0 = 0;
for i= 1:Nv 
    for j = 1:Nc-wantblockexogobs
        if (prova(j,i) ==0)
            num0 = num0+1;
        else
            variable_pos(j,i)= prova(j,i)-num0; 
        end 
    end
end
 

%Remove observable factors from space spanned by latent factors
[F0,CC,IC,Loading1] = factor_estim(YF,rplusM,normaliz); 
M = cols(OBS);

if optionrem == 1, %no removal of observables from space spanned by latent factors
    F = F0;
    r = rplusM;
    rplusM = r+M;
elseif optionrem == 0, %removal a la Boivin-Giannoni
    r = rplusM - M;

    e0 = 100; e1 = 1000;
    Ynew = YF; 
    while abs(e1-e0) >= .000001
        Yold = Ynew;
        e0 = e1;
        [CFY,CC,IC,Loading1] = factor_estim(Yold,r,normaliz); 
        lam = []; u = [];
        for i = 1:size(YF,2)
            res = ols(YF(:,i),[CFY OBS]);
            lam(:,i) = res.beta; 
            u(:,i) = res.resid; clear res;
        end
        e1 = sum(diag(u'*u))/size(YF,2)/T;
        abs(e1-e0)
        Ynew = YF - OBS*lam(r+1:end,:);
    end
    F = CFY;
    clear Ynew Yold CFY e1 e0 u res lam CC IC Loading1;
elseif optionrem == 2 %orthogonal complement (here for 1 observable)
    r = rplusM - M;
    Loading1 = [];
    for i = 1:size(OBS,2)
        res = ols(center(OBS(:,i)),F0); 
        Loading1(i,:) = res.beta';
    end
    F = F0*(null(Loading1));

%What if we assumed tv relation btw OBS and F0?
%     Loading1tv = [];
%     for i = 1:size(OBS,2)
%         Loading1tv = SE_tvreg(center(OBS(:,i)),F0,0); 
%     end
%     for t = 1:T
%         Ftv(t,:) = F0(t,:)*(null(Loading1tv(t,:)));
%     end
% 
end
clear YF;

%--> Compare Ftv and F and look at correlation btw the elements of Ftv after removal of OBS
% traceR2(F,Ftv)
% sort(vec(abs(tril(corrcoef(Ftv),-1))))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

X = [OBS F];
%X = [F OBS];
%How much of the total variance is explained by latent and observed factors?
% v =[]; for i = 1:N, res = ols(Y(:,i),[X ones(T,1)]); v(i) = var(res.yhat)/var(Y(:,i)); end, nanmean(v)  %how much is explained by the r latent and observed factors?

diagWW = diag(WW);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Save the  factors and the standardized Y
fMat = X; %rename to be used with existing code
yMat = Y;
cd ..\Results;

fmat12 = fMat;
%load FMAT
%save('FMAT', 'fmat09', 'fmat10', 'fmat11' , 'fmat12')
save(['fMatyMat_',IdentStrg], 'IdentStrg', 'fMat', 'yMat', 'nbobs', 'nv', 'codes', 'diagWW','volatype','vola_fun',...
    'opt_endtime','opt_starttime','wantblockexogobs', 'variable_names', 'countries', 'variable_pos', 'pos_fci');
cd ..\CodeData;
%note the IdentStrg it




