% Wolfgang Lemke, 29 May 2010
% Forecast error variance decomp for linear fct of a variable that follows
% a VAR

function [FEVD,FEV]=WL_AffVAR_FEVD(Kap,R,lambda,HMax,cumul);

% VAR Model:
% X_t = c + Kap X_{t-1} + R w_t,    w_t ~ N(0,I)
% Linear function
% y_t = lambda'X_t
% this may represent the companion form
% dimensions:
% X: M x 1; Kap: M x M; R: M x n; w: n x 1; lambda: M x 1; y: 1 x 1
% again n shocks, M variables in X
% forecast error e_{t+h} = lambda'(X_{t+h} - E_t X_{t+h})

% INPUT: 
% Kap, R, HMax max forec horizon, lambda
% compute FEVD for variables 1 to m (e.g. when VAR is companion form,
% one would like to have only the non-lagged elements of X_t
% cumul: cumulated FEVD (1) or not (0)

% OUTPUT: 
% FEVD: HMax x n
% (h,j) element: the fraction of y forec error variance (horizon h) which
% is attributabe to shock j 

[M,n]=size(R);
FEVD=NaN(HMax,n);
FEV=NaN(HMax,1);

AiCum=zeros(M,n);
BiSqCum=zeros(1,n);
BiCumSqCum=zeros(1,n);

for i=1:HMax; 
    Ai=Kap^(i-1)*R;
    AiCum=AiCum+Ai;
    Bi=lambda'*Ai;
    BiCum=lambda'*AiCum;
    
    BiSqCum=BiSqCum+Bi.^2;
    BiCumSqCum=BiCumSqCum+BiCum.^2;
    
    if cumul==1;
        vari=sum(BiCumSqCum,2);
        FEVD(i,:)=BiCumSqCum/vari;
        FEV(i)=vari;
    else;
        vari=sum(BiSqCum,2);
        FEVD(i,:)=BiSqCum/vari;
        FEV(i)=vari;
    end;
end;




