% Wolfgang Lemke
% 30 May 2010
% forecast error variance decomp for FAVAR with possible AR(1) errors
function FEVD=WL_FAVAR_FEVD(KapC,R,Lamb,sigVec,rhoVec,Cumul,HMax);

% The VAR in companion form
% X_t = c + Kap_1 X_{t-1} + [R; 0] w_t
% The cross_sect equation v
% y_t^v = d_v + Lamb_{v,.}X_t + u^v_t,  u^v_t = rho_v u^v_{t-1} + sig*eps_t,
% eps_t ~  N(0,1), w_t ~ N(0,I_G)

% INPUT:
% KapC: companion form Kappa, G*p x G*p
% R: G x G 
% Lamb: N x G
% sigVec: N x 1
% rhoVec: N x 1
% Cumul: N x 1
% HMax: max forecast horizon

% OUTPUT
% FEVD: forecast error variance decom, N x G x HMax

G=size(R,1);
p=size(KapC,1)/G;
N=size(Lamb,1);

FEVD=NaN(N,G+1,HMax);

for vv=1:N;
    KapAug=zeros(G*p+1,G*p+1); 
    KapAug(1:end-1,1:end-1)=KapC; KapAug(end,end)=rhoVec(vv);
    RAug=zeros(G*p+1,G+1); 
    RAug(1:G,1:G)=R; RAug(end,end)=sigVec(vv);
    lamb=Lamb(vv,:);
    lambAug=zeros(G*p+1,1); 
    lambAug(1:G)=lamb'; lambAug(end)=1;
    [FEVDloc,FEVloc]=WL_AffVAR_FEVD(KapAug,RAug,lambAug,HMax,Cumul(vv));
    % note that FEVDloc is HMax x G+1
    FEVD(vv,:,:)=FEVDloc';
end;


