% Wolfgang Lemke, April 2008
% Kalman filter for time-const and timevar. SSM
function [afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);
% I checked with Thomas state space code:
% for time-invariant model get exactly the same filtered states and llh

% NO DIM CHECK ETC , add LATER

%***********
% Model:
% complete...

%***********
% Input:
% complete...

%***********
% Output:
% complete...% 

%*******************
%   Dimensions 
%*******************
Tbig=size(yt,1);
nmeas=size(yt,2);
nsta=size(Kapt,1);

%*******************
%   Provide empty result matrices and multidim arrays 
%*******************
afis=NaN*ones(Tbig,nsta);
Pfis=NaN*ones(nsta,nsta,Tbig);
aprs=NaN*ones(Tbig,nsta);
Pprs=NaN*ones(nsta,nsta,Tbig);
yprs=NaN*ones(Tbig,nmeas);
vprs=yprs;
Fprs=NaN*ones(nmeas,nmeas,Tbig);
llh=-nmeas*Tbig/2*log(2*pi);

%*******************
%   Initialization
%*******************
if inista==1;
    afiloc=a0;
    Pfiloc=P0;
elseif inista==0;
    % only works  for stationary non-tv transition eq
    a0=inv(eye(nsta)-Kapt)*ct;
    vecP0=inv(eye(nsta^2)-kron(Kapt,Kapt))*vec(Rt*Qt*Rt');
    P0=reshape(vecP0,nsta,nsta);
    afiloc=a0;
    Pfiloc=P0;
end;
       
%*******************
%   Kalman filter, time-indep SSM
%*******************
if istv==0; 
    % if time-indep SSM, can set syst matrices once and for all
    Kap=Kapt;
    R=Rt;
    c=ct;
    Q=Qt;
    Z=Zt;
    d=dt;
    H=Ht;
for tt=1:Tbig;    
    % Prediction step
    aprloc=Kap*afiloc+c;
    Pprloc=Kap*Pfiloc*Kap'+R*Q*R';
    yprloc=Z*aprloc+d;
    Fprloc=Z*Pprloc*Z'+H;
    %Updating step
    if rcond(Fprloc) > 10^-12
       invFprloc=inv(Fprloc);
   else
       invFprloc=pinv(Fprloc);
       %dummy=1;
    end
    afiloc=aprloc+Pprloc*Z'*invFprloc*(yt(tt,:)'-yprloc);
    vprloc=yt(tt,:)'-yprloc;
    Pfiloc=Pprloc-Pprloc*Z'*invFprloc*Z*Pprloc;
    % log likelihood
    llh=llh-0.5*log(det(Fprloc))-0.5*vprloc'*invFprloc*vprloc;
    % insert into container matrices etc
    afis(tt,:)=afiloc';
    Pfis(:,:,tt)=Pfiloc;
    aprs(tt,:)=aprloc';
    Pprs(:,:,tt)=Pprloc;
    yprs(tt,:)=yprloc';
    vprs(tt,:)=vprloc';
    Fprs(:,:,tt)=Fprloc;
end;
end;
%*******************
%   Kalman filter, time-dep SSM
%*******************
if istv==1; 
for tt=1:Tbig;
    % Set tt-current system matrices
    Kap=Kapt(:,:,tt);
    R=Rt(:,:,tt);
    c=ct(tt,:)';
    Q=Qt(:,:,tt);
    Z=Zt(:,:,tt);
    d=dt(tt,:)';
    H=Ht(:,:,tt);
  
    % Prediction step
    aprloc=Kap*afiloc+c;
    Pprloc=Kap*Pfiloc*Kap'+R*Q*R';
    yprloc=Z*aprloc+d;
    Fprloc=Z*Pprloc*Z'+H;
    %Updating step
  if rcond(Fprloc) > 10^-12
       invFprloc=inv(Fprloc);
   else
       invFprloc=pinv(Fprloc);
       %dummy=1;
  end
    afiloc=aprloc+Pprloc*Z'*invFprloc*(yt(tt,:)'-yprloc);
    vprloc=yt(tt,:)'-yprloc;
    Pfiloc=Pprloc-Pprloc*Z'*invFprloc*Z*Pprloc;
    % log likelihood
    llh=llh-0.5*log(det(Fprloc))-0.5*vprloc'*invFprloc*vprloc;
    % insert into container matrices etc
    afis(tt,:)=afiloc';
    Pfis(:,:,tt)=Pfiloc;
    aprs(tt,:)=aprloc';
    Pprs(:,:,tt)=Pprloc;
    yprs(tt,:)=yprloc';
    vprs(tt,:)=vprloc';
    Fprs(:,:,tt)=Fprloc;
end;
end;

