% Wolfgang Lemke, April 2008
% Kalman DISTURBANCE smoother for time-const and timevar. SSM
function [epsSmo,etaSmo,VarepsSmo,VaretaSmo,rSmo,NSmo,uSmo,DSmo]=WL_Kalmandistsmo(Pprs,yprs,Fprs,Kapt,Zt,Ht,Rt,Qt,yt,istv);
% this follows the Durbin/Koopman book, sect 4.4, but adjusted for our
% timing convention
% also note that their K matrix differs from ours!

%*******************
%   Dimensions 
%*******************
[nsta,nsta,Tbig]=size(Pprs);
nmeas=size(yt,2);
if istv==1;
    [neta,neta,Tbig]=size(Qt); %the dim of the state innovs
else;
    [neta,neta]=size(Qt);
end;

%*******************
%   Provide empty result matrices and multidim arrays 
%*******************
etaSmo=NaN(Tbig,neta);
epsSmo=NaN(Tbig,nmeas);
VaretaSmo=NaN(neta,neta,Tbig);
VarepsSmo=NaN(nmeas,nmeas,Tbig);
rSmo=NaN(Tbig,nsta);
NSmo=NaN(nsta,nsta,Tbig);
uSmo=NaN(Tbig,nmeas);
DSmo=NaN(nmeas,nmeas,Tbig);

%*******************
%   Prepare for time-variant and time-invariant SSM
%*******************
if istv==0;
% unlike for the filter/pred algorith, here only the tv version is implemented (see below):
% note that for the non-tv SSM, the Kappas etc are given without time
% dim, i.e one object only, so they have to be 'blown up' in order to be
% read by the tv algo, i.e.:
% if ssm is time-invariant, make a sequence of the Kap matrices, and the
% other objects
    Kapts=zeros(nsta,nsta,Tbig);
    Zts=zeros(nmeas,nsta,Tbig);
    Hts=zeros(nmeas,nmeas,Tbig);
    Rts=zeros(nsta,neta,Tbig);
    Qts=zeros(neta,neta,Tbig);
    for tt=1:Tbig;
        Kapts(:,:,tt)=Kapt;
        Zts(:,:,tt)=Zt;
        Hts(:,:,tt)=Ht;
        Rts(:,:,tt)=Rt;
        Qts(:,:,tt)=Qt;
    end;
else;
    Kapts=Kapt;
    Zts=Zt;
    Hts=Ht;
    Rts=Rt;
    Qts=Qt;
end;

%*******************
%   Conduct the residual smoothing, starting from the end
%*******************
% Hypothetical value for T+1;
rLoc=zeros(nsta,1);
NLoc=zeros(nsta,nsta);

% At time t, KapLoc is always Kap(t+1). 
% For T, KapLoc, referring to T+1 could be set to anything as multiplied by N, r, so not rel for t=T anyway
for tt=fliplr(1:1:Tbig); % go backwards
    % the syst matrices for t (except Kap for t+1)
    if tt==Tbig;
        KapLoc=zeros(nsta,nsta);
    else;
        KapLoc=Kapts(:,:,tt+1);
    end;
    ZLoc=Zts(:,:,tt);
    HLoc=Hts(:,:,tt);
    RLoc=Rts(:,:,tt);
    QLoc=Qts(:,:,tt);
    PprLoc=Pprs(:,:,tt);
    FprLoc=Fprs(:,:,tt);
    if abs(det(FprLoc))<eps;
        invFprLoc=pinv(FprLoc);
    else;
        invFprLoc=inv(FprLoc);
    end;
    KLoc=KapLoc*PprLoc*ZLoc'*invFprLoc; % note, this is the D+K K,different from mine
    LLoc=KapLoc-KLoc*ZLoc;
    vLoc=yt(tt,:)'-yprs(tt,:)';
    
    epsSmoLoc=HLoc*(invFprLoc*vLoc-KLoc'*rLoc); % note this is the r of period t+1 
    uSmoLoc=invFprLoc*vLoc-KLoc'*rLoc;
    VarepsSmoLoc=HLoc-HLoc*(invFprLoc+KLoc'*NLoc*KLoc)*HLoc; % note this is the N of period t+1 
    DSmoLoc=invFprLoc+KLoc'*NLoc*KLoc;
    rLoc=ZLoc'*invFprLoc*vLoc+LLoc'*rLoc; % now the r for period t is computed
    NLoc=ZLoc'*invFprLoc*ZLoc+LLoc'*NLoc*LLoc; % now the r for period t is computed
    etaSmoLoc=QLoc*RLoc'*rLoc;
    VaretaSmoLoc=QLoc-QLoc*RLoc'*NLoc*RLoc*QLoc;
      
    epsSmo(tt,:)=epsSmoLoc';
    etaSmo(tt,:)=etaSmoLoc';
    uSmo(tt,:)=uSmoLoc';    
    rSmo(tt,:)=rLoc';
    VarepsSmo(:,:,tt)=VarepsSmoLoc;
    VaretaSmo(:,:,tt)=VaretaSmoLoc;
    DSmo(:,:,tt)=DSmoLoc;
    NSmo(:,:,tt)=NLoc; 
end;
