% Wolfgang Lemke, June 2009
% function that takes a group (cell) of states at a time and constructs
% system matrices of a tv SVAR
function [P, KapArray, KapTildeArray, KapTildeComp]=WL_TVPVAR_StateGroup2SVAR(CellOfStates);

% the model is (ignoring intercept):
% P_t X_t = Kap_{1,t} X_{t-1} + ... + Kap_{p,t} X_{t-p} + R eps_t
% the Kap matrices are full
% P_t is (example for G=3):
% P_t = [1          0           0;
%        P_{21,t}   1           0;
%        P_{31,t}   P_{32,t}    1];
%
%
% OUTPUT:
% P: P_t
% KapArray: dimension G x G x p, array containing the Kap_{1,t} to
% Kap_{p,t}
% KapTildeArray: dimension G x G x p, array containing the inv(P_t)*Kap_{1,t} to
% inv(P_t)*Kap_{p,t}, i.e. 
% KapTildeComp: autoregr matrix of reduced-form VAR in companion form (G=3 example):
% [KapTilde_1 KapTilde_2 KapTilde_3 ;
%    I            0             0   ;    
%    0            I             0   ;]    
G=size(CellOfStates,1);
p=size(CellOfStates{1},1)/G;

P=eye(G);
KapArray=NaN(G,G,p);
KapTildeArray=NaN(G,G,p);

for gg=1:G;
    stateLoc=CellOfStates{gg};
    KElements=stateLoc(gg:end);
    KSorted=reshape(KElements,G,p); % each col is the K-elements belonging to one lag 
    for pp=1:p
        KapArray(gg,:,pp)=KSorted(:,pp)';
    end;
    if gg>1;
        P(gg,1:gg-1)=stateLoc(1:gg-1)'; %this has changed on 4 June 2010: now it is assumed that the state vector directly contains the elements of P and not the negative of those
    end;
end;
for pp=1:p;
    KapTildeArray(:,:,pp)=inv(P)*squeeze(KapArray(:,:,pp));
end;
KapTildeComp=[];
for pp=1:p;
    KapTildeComp=[KapTildeComp squeeze(KapTildeArray(:,:,pp))];
end;
if p>1;
KapTildeComp=[KapTildeComp; [eye((p-1)*G) zeros((p-1)*G,G)]];
end;
    