% Wolfgang Lemke, 29 May 2010
% Forecast error variance decomp for VAR

function [FEVD,FEV]=WL_VAR_FEVD(Kap,R,m,HMax,cumul);

% VAR Model:
% X_t = c + Kap X_{t-1} + R w_t,    w_t ~ N(0,I)
% this may represent the companion form
% dimensions:
% X: M x 1; Kap: M x M; R: M x n; w: n x 1;
% i.e. n shocks, M variables
% forecast error e_{t+h} = X_{t+h} - E_t X_{t+h}

% INPUT: 
% Kap, R, HMax max forec horizon, m<M
% m: compute FEVD for variables 1 to m (e.g. when VAR is companion form,
% one would like to have only the non-lagged elements of X_t
% cumul: cumulated FEVD (1) or not (0)

% OUTPUT: 
% FEVD: m x n x HMax
% (r,j,h) element: the fraction of rth variable forec error variance (horizon h) which
% is attributabe to shock j 

[M,n]=size(R);
FEVD=NaN(m,n,HMax);
FEV=NaN(HMax,M);

AiSqCum=zeros(M,n);
AiCum=zeros(M,n);
AiCumSqCum=zeros(M,n);
for i=1:HMax;
    Ai=Kap^(i-1)*R;
    AiSq=Ai.^2;
    AiSqCum=AiSqCum+AiSq;
    AiCum=AiCum+Ai;
    AiCumSq=AiCum.^2;
    AiCumSqCum=AiCumSqCum+AiCumSq;
    if cumul==1;
        vars=sum(AiCumSqCum,2);
        VDloc=AiCumSqCum./(vars*ones(1,n));
        FEV(i,:)=vars';
    else;
        vars=sum(AiSqCum,2);
        VDloc=AiSqCum./(vars*ones(1,n));
        FEV(i,:)=vars';
    end;
    FEVD(:,:,i)=VDloc(1:m,:);
end;




