% Eickmeier/Lemke/Marcellino (2013)
% Code for TV-FAVAR analysis
% Last modification: 31 Oct 2013


% INPUT: 
% ZZ: G-dim cell, each entry is the vola series (possibly constant vector
% of ones) for the g-th equation. Note that they can differ in dimension


% the initial a0 corresponding to the initial Kap, P etc is based on whole
%-sample OLS

function Results=WL_tvVAR(KapType,PType,KapFix,PFix,cFix,XX,ZZ,p,P0Scale,epstol,optalgo,volatype);

wantRobustifyGARCH_estimation=1; 
%if =1, then do robustified GARCH estimation with several (see below) starting values
% if =0, then do GARCH just based on one stdd error
nbGARCHsv=2; % number of starting values for GARCH

[T,G]=size(XX);
global opt_starttime
% provide empty containers

% For RW and other hyper pars parameter estimates
RWstdhatCells=cell(G,1);
a0Cells=cell(G,1);
P0Cells=cell(G,1);

khatCells=cell(G,1);
se_khatCells=cell(G,1);

InnovVolaFortv=NaN(T,G);

if wantRobustifyGARCH_estimation==1;
    GARCH_StartingValuesCells=cell(G,1);
    GARCH_negLlhsCells=cell(G,1);
    GARCH_EstimatedParsCells=cell(G,1);
end;

% For tv parameter estimates
tvKapRestr=NaN(T,G,G,p); %container for restricted  (stationarity  condition) version
tvPRestr=NaN(T,G,G);
tvKapTildeCompRestr=NaN(T,p*G,p*G);
tvKapTildeCompMaxEigRestr=NaN(T,1);

tvKapUnRestr=NaN(T,G,G,p); %container for restricted  (stationarity  condition) version
tvPUnRestr=NaN(T,G,G);
tvKapTildeCompUnRestr=NaN(T,p*G,p*G);
tvKapTildeCompMaxEigUnRestr=NaN(T,1);

tvKapRestrSmo=NaN(T,G,G,p); %container for restricted  (stationarity  condition) version
tvPRestrSmo=NaN(T,G,G);
tvKapTildeCompRestrSmo=NaN(T,p*G,p*G);
tvKapTildeCompMaxEigRestrSmo=NaN(T,1);

tvKapUnRestrSmo=NaN(T,G,G,p); %container for restricted  (stationarity  condition) version
tvPUnRestrSmo=NaN(T,G,G);
tvKapTildeCompUnRestrSmo=NaN(T,p*G,p*G);
tvKapTildeCompMaxEigUnRestrSmo=NaN(T,1);

tvResidSmoRestr=NaN(T,G);
tvResidSmoUnRestr=NaN(T,G);

% % Construct  block of regressors
XBlock=XX(p+1:T,:);
XLagsBlock=[];
for pp=1:p;
    XLagsBlock=[XLagsBlock XX(p+1-pp:T-pp,:)];
end;


% Estimate equation by equation

for gg=1:G;
    disp(['_________________________________________________________ ']);
    disp(['Estimate ' num2str(gg) 'th of ' num2str(G) ' VAR equations']);
    disp(['_________________________________________________________ ']);
    pause(1);
    %prepare regressor matrix for respective equation 
    y=XBlock(:,gg);
    if gg==1;
        XContemp=[];
        CoeffTypes=[];
        PreCoeffs=[];
    else;
        XContemp=-XBlock(:,1:gg-1); % NOTE THE minus sign!
        CoeffTypes=PType(gg,1:gg-1);
        PreCoeffs=PFix(gg,1:gg-1);
    end;
    X=[XContemp XLagsBlock];
    for pp=1:p;
        CoeffTypes=[CoeffTypes KapType(gg,:,pp)];
        PreCoeffs=[PreCoeffs KapFix(gg,:,pp)];
    end;
    % coeffTypes is now an gg-1+p*G row vector consisting of {0,1,2}
    
    % Subtract the intercepts and the known coeffs*X from y
    % find pos of predet. parameters
    FixPos=find(CoeffTypes==0);
    EstPosConstPar=find(CoeffTypes==1);
    EstPosTVPar=find(CoeffTypes==2);
    EstPos=find(CoeffTypes>0);
    CoeffTypesEst=CoeffTypes(EstPos);
    yber=y-cFix(gg)*ones(T-p,1)-X(:,FixPos)*PreCoeffs(FixPos)';
    X1=X(:,EstPos);
    tvInd=(CoeffTypesEst==2);
    
    RWstdini=0.1*ones(sum(tvInd),1);
    % do the estimation

    % this setting of a0, P0, ...
    % a0=[zeros(gg-1,1); 0.1*ones(sum(tvind)-gg+1,1)];
    % P0=P0Scale*eye(sum(tvind));
    % ..., or: a0, P0 via OLS:
    [betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y,X1);
    InnovVolaFornontv(gg)=sqrt(sum(resid.*resid))/sqrt(length(y)-length(betahat));
    %a0=betahat(1:end); % without intercept in VAR
    %P0=P0Scale*Varbetahat(1:end,1:end); % without intercept in VAR
    
    % get the vola series of this equation
    if volatype<5; 
    ZZgg=ZZ{gg};
    nbZ=size(ZZgg,2);
    % and starting values for the loadings
    ksv=0.1*ones(nbZ,1);
    end;

    a0=NaN; % this makes the a0 in the function estimate it by OLS on whole sample
    P0=NaN;
 
if volatype<5    
    if strcmp(opt_starttime,'1975')
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,khat,se_khat,parfilt,parsmo,Xnew,stdepsini,a0Out,P0Out,MiscOut]=WL_tvreg_spv_sepvolaregr_CISSUS(yber,X1,tvInd,a0,P0,P0Scale,RWstdini,epstol,optalgo, ksv, ZZgg(p+1:end,:), volatype);
    else
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,khat,se_khat,parfilt,parsmo,Xnew,stdepsini,a0Out,P0Out,MiscOut]=WL_tvreg_spv_sepvolaregr(yber,X1,tvInd,a0,P0,P0Scale,RWstdini,epstol,optalgo, ksv, ZZgg(p+1:end,:), volatype);
    end
    % the filtered states from here are already correpsonding to the tv
    % params in the respective rows of the SVAR, however, they are jointly
    % (my concert KF) estimated below again.. Hence, the job of
    % WL_tvreg_spv_sepvolaregr here is mainly to provide the estimated
    % hyper params
elseif volatype==5; %GARCH VERSION
    if wantRobustifyGARCH_estimation==1;
        %work with seven different starting values
%         mystep=(0.7-0.1)/(nbGARCHsv-1);
%         ksv_vec1=0.1*ones(1,nbGARCHsv);
%         ksv_vec2=0.1:mystep:0.7;
%         ksv_vec3=0.8-ksv_vec2;

        
        mystep=(0.75-0.05)/(nbGARCHsv-1); % CHANGE STEP
        ksv_vec1=0.1*ones(1,nbGARCHsv);
        ksv_vec2=0.05:mystep:0.75;
        ksv_vec3=0.8-ksv_vec2;
        
        negllhvec=NaN(nbGARCHsv,1);
        khat_vec=NaN(3,nbGARCHsv);
        
        for vv=1:nbGARCHsv;
            disp(['WL: GARCH estimation: starting value ',num2str(vv),' of ',num2str(nbGARCHsv)]);
            ksv=NaN(3,1);
            ksv(1)=ksv_vec1(vv);
            ksv(2)=ksv_vec2(vv);
            ksv(3)=ksv_vec3(vv);
            try;
                %try to optimize the likelihood
                [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,khat,se_khat,parfilt,parsmo,Xnew,stdepsini,a0Out,P0Out,MiscOut]=WL_tvreg_GARCHvola(yber,X1,tvInd,a0,P0,P0Scale,RWstdini,epstol,optalgo,ksv);
                if isreal(MiscOut.negllh);
                    negllhvec(vv)=MiscOut.negllh;
                    khat_vec(:,vv)=khat;
                else;
                    negllhvec(vv)=NaN;
                end;
            catch;
                disp(['WL: GARCH estimation: cannot optimize llh with starting value ',num2str(vv),' not successful']);
           end;
        end;
         
        disp('WL: GARCH estimation: now run again from best starting value');
        [myaux,Sort_Index]=sort(negllhvec);
        %(o/w we would have needed to
        %save all results from all run)
        vvstar=Sort_Index(1);
        ksv(1)=ksv_vec1(vvstar);
        ksv(2)=ksv_vec2(vvstar);
        ksv(3)=ksv_vec3(vvstar);
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,khat,se_khat,parfilt,parsmo,Xnew,stdepsini,a0Out,P0Out,MiscOut]=WL_tvreg_GARCHvola(yber,X1,tvInd,a0,P0,P0Scale,RWstdini,epstol,optalgo, ksv);
        
        % store the results from the GARCH robustness analysis
        
        GARCH_StartingValuesCells{gg}=[ksv_vec1; ksv_vec2; ksv_vec3];
        GARCH_negLlhsCells{gg}=negllhvec';
        GARCH_EstimatedParsCells{gg}=khat_vec;
        
        
    else;
        %starting values
        ksv=ones(3,1);
        ksv(1)=0.1; % is overwritten within the function
        ksv(2)=0.1; % parameter of lagged squared resid in GARCH
        ksv(3)=0.7; % parameter of lagged h in GARCH
        [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,khat,se_khat,parfilt,parsmo,Xnew,stdepsini,a0Out,P0Out,MiscOut]=WL_tvreg_GARCHvola(yber,X1,tvInd,a0,P0,P0Scale,RWstdini,epstol,optalgo, ksv);
    end;
else;
    disp('WL error: this volatility type not supported');
end;
    % Write estimated random walk parameters to container
    AuxRWstd=zeros(gg-1+p*G,1);
    AuxRWstd(EstPosTVPar)=RWstdhat;
    RWstdhatCells{gg}=AuxRWstd;
    
    khatCells{gg}=khat;
    se_khatCells{gg}=se_khat;
    
    % Write estimated INITIAL states to container (based on OLS)
    Auxa0=zeros(gg-1+p*G,1);
    Auxa0(EstPosTVPar)=a0Out;
    Auxa0(EstPosConstPar)=nontvparhat;
    Auxa0(FixPos)=PreCoeffs(FixPos);
    a0Cells{gg}=Auxa0;
    
    AuxP0=zeros(gg-1+p*G,gg-1+p*G);
    AuxP0(EstPosTVPar,EstPosTVPar)=P0Out;
    P0Cells{gg}=AuxP0;
    
    InnovVolaFortv(p+1:end,gg)=stdepshat;
end;

%______________________________________________________
%   Compute the constant-param VAR objects
%______________________________________________________
[nontvP, nontvKap, nontvKapTilde, nontvKapTildeComp]=WL_TVPVAR_StateGroup2SVAR(a0Cells);

%______________________________________________________
%   Now that the hyper-parameters are known from the equationwise
%   estimation, run the G SSMs in concert and do updating only if
%   stability condition is satisfied. 
%______________________________________________________
% Generate the groups (cells!) of SSMs:
% - the a0Cells and P0Cells are already given (see above)
[yt,dt,Zt,Ht,ct,Kapt,Rt,Qt]=WL_TVPVAR_MakeSSMGroup(XX,RWstdhatCells,p,cFix,InnovVolaFortv);
% do the group estimation
inista=1; % use given a0 P0 for initialization
istv=1;
RestrFuncStr='WL_TVPVAR_CheckStat'; % name of restriction function
[afisRestr,PfisRestr,yprsRestr,FprsRestr,aprsRestr,PprsRestr,llhRestr,RestrSatTSRestr,asmosRestr,PsmosRestr,epsSmoRestr,etaSmoRestr,VarepsSmoRestr,VaretaSmoRestr,rSmoRestr,NSmoRestr,uSmoRestr,DSmoRestr]=WL_KalmanGroupRestr(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0Cells,P0Cells,inista,istv,RestrFuncStr);
RestrFuncStr='WL_TVPVAR_AlwaysStat'; % name of restriction function: this one is trivial is allows all state combis
[afisUnRestr,PfisUnRestr,yprsUnRestr,FprsUnRestr,aprsUnRestr,PprsUnRestr,llhUnRestr,RestrSatTSUnRestr,asmosUnRestr,PsmosUnRestr,epsSmoUnRestr,etaSmoUnRestr,VarepsSmoUnRestr,VaretaSmoUnRestr,rSmoUnRestr,NSmoUnRestr,uSmoUnRestr,DSmoUnRestr]=WL_KalmanGroupRestr(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0Cells,P0Cells,inista,istv,RestrFuncStr);
% the outputs afisRestr, etc are all cells, i.e. groups of filtered states

for tt=p+1:T;
    % make local cell of states
    afisCellLocRestr=cell(G,1);
    afisCellLocUnRestr=cell(G,1);
    
    asmosCellLocRestr=cell(G,1);
    asmosCellLocUnRestr=cell(G,1);
  
    for gg=1:G;
        afisCellLocRestr{gg}=(afisRestr{gg}(tt-p,:))'; %NOTE: afis  is shorter, it does not have the leading p NaNs
        afisCellLocUnRestr{gg}=(afisUnRestr{gg}(tt-p,:))'; %NOTE: afis   is shorter, it does not have the leading p NaNs
        
        asmosCellLocRestr{gg}=(asmosRestr{gg}(tt-p,:))'; %NOTE: afis  is shorter, it does not have the leading p NaNs
        asmosCellLocUnRestr{gg}=(asmosUnRestr{gg}(tt-p,:))'; %NOTE: afis   is shorter, it does not have the leading p NaNs
    end;
    
    [PRestr, KapArrayRestr, KapTildeArrayRestr, KapTildeCompRestr]=WL_TVPVAR_StateGroup2SVAR(afisCellLocRestr);
    tvKapRestr(tt,:,:,:)=KapArrayRestr;
    tvPRestr(tt,:,:)=PRestr;
    tvKapTildeCompRestr(tt,:,:)=KapTildeCompRestr;
    tvKapTildeCompMaxEigRestr(tt)=max(abs(eig(KapTildeCompRestr)));
    
    [PUnRestr, KapArrayUnRestr, KapTildeArrayUnRestr, KapTildeCompUnRestr]=WL_TVPVAR_StateGroup2SVAR(afisCellLocUnRestr);
    tvKapUnRestr(tt,:,:,:)=KapArrayUnRestr;
    tvPUnRestr(tt,:,:)=PUnRestr;
    tvKapTildeCompUnRestr(tt,:,:)=KapTildeCompUnRestr;
    tvKapTildeCompMaxEigUnRestr(tt)=max(abs(eig(KapTildeCompUnRestr)));
    
    [PRestrSmo, KapArrayRestrSmo, KapTildeArrayRestrSmo, KapTildeCompRestrSmo]=WL_TVPVAR_StateGroup2SVAR(asmosCellLocRestr);
    tvKapRestrSmo(tt,:,:,:)=KapArrayRestrSmo;
    tvPRestrSmo(tt,:,:)=PRestrSmo;
    tvKapTildeCompRestrSmo(tt,:,:)=KapTildeCompRestrSmo;
    tvKapTildeCompMaxEigRestrSmo(tt)=max(abs(eig(KapTildeCompRestrSmo)));
    
    [PUnRestrSmo, KapArrayUnRestrSmo, KapTildeArrayUnRestrSmo, KapTildeCompUnRestrSmo]=WL_TVPVAR_StateGroup2SVAR(asmosCellLocUnRestr);
    tvKapUnRestrSmo(tt,:,:,:)=KapArrayUnRestrSmo;
    tvPUnRestrSmo(tt,:,:)=PUnRestrSmo;
    tvKapTildeCompUnRestrSmo(tt,:,:)=KapTildeCompUnRestrSmo;
    tvKapTildeCompMaxEigUnRestrSmo(tt)=max(abs(eig(KapTildeCompUnRestrSmo)));
 end;

Results.Restr.tvKap=tvKapRestr;
Results.Restr.tvP=tvPRestr;
Results.Restr.tvKapTildeComp=tvKapTildeCompRestr;
Results.Restr.tvKapTildeCompMaxEig=tvKapTildeCompMaxEigRestr;

Results.UnRestr.tvKap=tvKapUnRestr;
Results.UnRestr.tvP=tvPUnRestr;
Results.UnRestr.tvKapTildeComp=tvKapTildeCompUnRestr;
Results.UnRestr.tvKapTildeCompMaxEig=tvKapTildeCompMaxEigUnRestr;

Results.RestrSmo.tvKap=tvKapRestrSmo;
Results.RestrSmo.tvP=tvPRestrSmo;
Results.RestrSmo.tvKapTildeComp=tvKapTildeCompRestrSmo;
Results.RestrSmo.tvKapTildeCompMaxEig=tvKapTildeCompMaxEigRestrSmo;

Results.UnRestrSmo.tvKap=tvKapUnRestrSmo;
Results.UnRestrSmo.tvP=tvPUnRestrSmo;
Results.UnRestrSmo.tvKapTildeComp=tvKapTildeCompUnRestrSmo;
Results.UnRestrSmo.tvKapTildeCompMaxEig=tvKapTildeCompMaxEigUnRestrSmo;

Results.RestrSmo.epsSmo=epsSmoRestr;
Results.RestrSmo.etaSmo=etaSmoRestr;
Results.RestrSmo.VarepsSmo=VarepsSmoRestr;
Results.RestrSmo.VaretaSmo=VaretaSmoRestr;
Results.RestrSmo.rSmo=rSmoRestr;
Results.RestrSmo.NSmo=NSmoRestr;
Results.RestrSmo.uSmo=uSmoRestr;
Results.RestrSmo.DSmo=DSmoRestr;

Results.UnRestrSmo.epsSmo=epsSmoUnRestr;
Results.UnRestrSmo.etaSmo=etaSmoUnRestr;
Results.UnRestrSmo.VarepsSmo=VarepsSmoUnRestr;
Results.UnRestrSmo.VaretaSmo=VaretaSmoUnRestr;
Results.UnRestrSmo.rSmo=rSmoUnRestr;
Results.UnRestrSmo.NSmo=NSmoUnRestr;
Results.UnRestrSmo.uSmo=uSmoUnRestr;
Results.UnRestrSmo.DSmo=DSmoUnRestr;

Results.InnovVola=InnovVolaFortv;

Results.NonTV.nontvP=nontvP;
Results.NonTV.nontvKap=nontvKap;
Results.NonTV.nontvKapTilde=nontvKapTilde;
Results.NonTV.nontvKapTildeComp=nontvKapTildeComp;
Results.NonTV.InnovVolaFornontv=InnovVolaFornontv;

Results.States.asmosRestr=asmosRestr;  %NEWCI have to save also these
Results.States.PsmosRestr=PsmosRestr;  %NEWCI have to save also these

Results.FacForecastsRestr.yprs=yprsRestr;
Results.FacForecastsRestr.Fprs=FprsRestr;

Results.FacForecastsUnRestr.yprs=yprsUnRestr;
Results.FacForecastsUnRestr.Fprs=FprsUnRestr;

Results.Hyperpars.khat=khatCells;
Results.Hyperpars.se_khat=se_khatCells;

if wantRobustifyGARCH_estimation==1;
    Results.GARCH_Robustness.GARCH_StartingValues=GARCH_StartingValuesCells;
    Results.GARCH_Robustness.GARCH_negLlhs=GARCH_negLlhsCells;
    Results.GARCH_Robustness.GARCH_EstimatedPars=GARCH_EstimatedParsCells;
    
     
    %print out the GARCH robustness results:
    disp('=============================================');
    disp('Diagnostics of robust (multiple starting values) GARCH estimation');
    disp('=============================================');
    for gg=1:G;
        disp(['VAR equation ',num2str(gg),':']);
        disp('starting values');
        disp(num2str(Results.GARCH_Robustness.GARCH_StartingValues{gg}));
        disp('neg. likelihoods');
        disp(num2str(Results.GARCH_Robustness.GARCH_negLlhs{gg}));
        disp('estimated parameters');
        disp(num2str(Results.GARCH_Robustness.GARCH_EstimatedPars{gg}));
        disp('------------------------------------');
    end;
end;

        
