% Wolfgang Lemke March 2009.
% This version Oct 2010: more output: yprs and Fprs are new
% function that estimates regression model with tv parameters

function [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,parfilt,parsmo,parfiltVari,parsmoVari,Xnew,stdepsini,yprs,Fprs]=WL_tvreg(y,X,tvind,a0,P0,P0Scale,RWstdini,nontvini,epstol,optalgo)

% INPUT
%... complete

% OUTPUT:
% ...

%____________________________________________________________________
%   Partition X
%____________________________________________________________________
% Resort the columns of X, s.t. first the variables, for which regression
% coefficients are constant, appear, and then the others
% Xnew = [X^A X^B];
[Tbig,cX]=size(X);
allcol=(1:cX)';
nontvind=(tvind-1).^2;
nontvcol=selif(allcol,nontvind); tvcol=selif(allcol,tvind);
XA=X(:,nontvcol); XB=X(:,tvcol);
[Tbig,cXA]=size(XA); [Tbig,cXB]=size(XB); 
Xnew=[XA XB];

%____________________________________________________________________
%   Constant param regression and starting values
%____________________________________________________________________
% Run constant param regression to get
% i) starting values for the ML estimates of constant parameters
% ii) starting values for the  states representing the tv parameters a0
% iii) the variance of the latter P0
% ... unless they are given over as input to the function directly 
% iv) the variance of the regression innovation
[betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y,Xnew);
if nontvini==0;
   nontvini=betahat(1:cXA);
end; 
if isnan(a0);
   a0=betahat(cXA+1:end);
end;
if isnan(P0);
   P0=P0Scale*Varbetahat(cXA+1:end,cXA+1:end);
else;
   P0=P0*P0Scale;
end;
stdepsini=std(resid);
start_pars=[nontvini; RWstdini; stdepsini];

%____________________________________________________________________
%   Maximize likelihood
%____________________________________________________________________
%     
if optalgo==1; %simplex
    mytol=epstol;
    mymaxiters=1500;
    options = optimset('Display','iter','MaxFunEvals',9000000,'MaxIter',mymaxiters,'TolFun',mytol,'TolX',mytol,'TolCon',mytol,'HessUpdate','bfgs');
    optpars=fminsearch(@(pars) objfctloc(pars,a0,P0,y,XA,XB),start_pars,options);
elseif optalgo==2; % BFGS
    mymaxitersBFGS=150;
    mytolBFGS=epstol;
    optionsBFGS=optimset('Display','iter','MaxFunEvals',100000,'MaxIter',mymaxitersBFGS,'TolFun',mytolBFGS,'TolX',mytolBFGS,'TolCon',mytolBFGS,'HessUpdate','bfgs');
    [optpars,neglllh,exflag]=fminunc(@(pars)objfctloc(pars,a0,P0,y,XA,XB),start_pars,optionsBFGS);
end; %
% 
%optpars=start_pars;
%____________________________________________________________________
%   Filtering given optimal parameters
%____________________________________________________________________
yt=y;
[Tbig,cA]=size(XA);
[Tbig,cB]=size(XB);
gam=optpars(1:cA);
dt=XA*gam;
Zt=reshape(XB',1,cB,Tbig); % WL_Kalman requires this to be an array
Ht=reshape(optpars(end)^2*ones(Tbig,1),1,1,Tbig);
ct=zeros(Tbig,cB);
Kapt=NaN(cB,cB,Tbig); %does the construction of Kapt, Rt, Qt go faster?
Rt=Kapt;
Qt=Kapt;
Qmat=(diag(optpars(cA+1:end-1)))^2;
for tt=1:Tbig;
    Kapt(:,:,tt)=eye(cB);
    Rt(:,:,tt)=eye(cB);
    Qt(:,:,tt)=Qmat;
end;
inista=1; % use the given a0, P0
istv=1;
[afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);    
% now run smoother
[asms,Psms]=WL_Kalmansmo(afis,Pfis,aprs,Pprs,Kapt,istv);


%____________________________________________________________________
%   Creation of further results
%____________________________________________________________________
nontvparhat=optpars(1:cA);
RWstdhat=optpars(cA+1:end-1);
nontvparse=NaN(cA,1);
RWstdse=NaN(cB,1);
parfilt=afis;
parsmo=asms;
parfiltVari=Pfis;
parsmoVari=Psms;
stdepshat=abs(optpars(end));

%return;
%____________________________________________________________________
%   Nested function: objective function for ML estimation
%____________________________________________________________________
    function negllh=objfctloc(pars,a0,P0,y,XA,XB)
    % set up state space model
    yt=y;
    [Tbig,cA]=size(XA);
    [Tbig,cB]=size(XB);
    gam=pars(1:cA);
    dt=XA*gam;
    Zt=reshape(XB',1,cB,Tbig); % WL_Kalman requires this to be an array
    Ht=reshape(pars(end)^2*ones(Tbig,1),1,1,Tbig);
    ct=zeros(Tbig,cB);
    Kapt=NaN(cB,cB,Tbig); %does the construction of Kapt, Rt, Qt go faster?
    Rt=Kapt;
    Qt=Kapt;
    Qmat=(diag(pars(cA+1:end-1)))^2;
    for t=1:Tbig;
        Kapt(:,:,t)=eye(cB);
        Rt(:,:,t)=eye(cB);
        Qt(:,:,t)=Qmat;
    end;
    inista=1; % use the given a0, P0
    istv=1;
    [afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);
    negllh=-llh;
    end %end nested function

end %main function

    