% Wolfgang Lemke May 2009.
% This version Oct 2010: more output: yprs and Fprs are new
% function that estimates regression model with tv parameters
% same as WL_tvreg but with AR(1) error
% to be done: join all these functions together

function [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,parfilt,parsmo,parfiltVari,parsmoVari,Xnew,stdepsini,rhohat,yprs,Fprs]=WL_tvreg_AR1error(y,X,tvind,a0,P0,P0scale,RWstdini,nontvini,epstol,optalgo)

% Model is
% y_t  = XA_t'beta_const  + XB_t'beta_t + u_t
% u_t = rho u_{t-1} + eps_t, eps_t \sim N(0,sigma^2)

% INPUT
%... complete

% OUTPUT:
% ...
% stdepshat: this now the ML estimate sigma
% 
% TO DO

%____________________________________________________________________
%   Partition X
%____________________________________________________________________
% Resort the columns of X, s.t. first the variables, for which regression
% coefficients are constant, appear, and then the others
% Xnew = [X^A X^B];
[Tbig,cX]=size(X);
allcol=(1:cX)';
nontvind=(tvind-1).^2;
nontvcol=selif(allcol,nontvind); tvcol=selif(allcol,tvind);
XA=X(:,nontvcol); XB=X(:,tvcol);
[Tbig,cXA]=size(XA); [Tbig,cXB]=size(XB); 
Xnew=[XA XB];

%____________________________________________________________________
%   Constant param regression and starting values
%____________________________________________________________________
% Run constant param regression to get
% i) starting values for the ML estimates of constant parameters
% ii) starting values for the  states representing the tv parameters a0
% iii) the variance of the latter P0
% ... unless they are given over as input to the function directly 
% iv) the variance of the regression innovation
[betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y,Xnew);
if nontvini==0;
   nontvini=betahat(1:cXA);
end; 
if isnan(a0);
   a0=[betahat(cXA+1:end); 0];
end;
if isnan(P0);
   P0=zeros(length(a0),length(a0)); 
   P0(1:end-1,1:end-1)=P0scale*Varbetahat(cXA+1:end,cXA+1:end);
else;
   P0=P0*P0scale;
end;
% NOTE: the last element of P0 will be overwritten anyway in the function

% estimate an Ar1 on the OLS residuals
[rho_u,tval_u,resid_u,Rsq_u,Varbetahat_u]=WL_ols(resid(2:end),resid(1:end-1));

stdepsini=std(resid_u);
start_pars=[nontvini; RWstdini; stdepsini; rho_u];

%____________________________________________________________________
%   Maximize likelihood
%____________________________________________________________________
%     
if optalgo==1; %simplex
    mytol=epstol;
    mymaxiters=1500;
    options = optimset('Display','iter','MaxFunEvals',9000000,'MaxIter',mymaxiters,'TolFun',mytol,'TolX',mytol,'TolCon',mytol,'HessUpdate','bfgs');
    optpars=fminsearch(@(pars) objfctloc(pars,a0,P0,y,XA,XB),start_pars,options);
elseif optalgo==2; % BFGS
    mymaxitersBFGS=150;
    mytolBFGS=epstol;
    optionsBFGS=optimset('Display','iter','MaxFunEvals',100000,'MaxIter',mymaxitersBFGS,'TolFun',mytolBFGS,'TolX',mytolBFGS,'TolCon',mytolBFGS,'HessUpdate','bfgs');
    [optpars,neglllh,exflag]=fminunc(@(pars)objfctloc(pars,a0,P0,y,XA,XB),start_pars,optionsBFGS);
end; %
% 
%optpars=start_pars;
%____________________________________________________________________
%   Filtering given optimal parameters
%____________________________________________________________________
yt=y;
P0(end,end)=(optpars(end-1)^2)/(1-optpars(end)^2);
[Tbig,cA]=size(XA);
[Tbig,cB]=size(XB);
gam=optpars(1:cA);
dt=XA*gam;
XB1=[XB ones(Tbig,1)]; % the ones are needed to pick the error as last state vector element
Zt=reshape(XB1',1,cB+1,Tbig); % WL_Kalman requires this to be an array
Ht=reshape(zeros(Tbig,1),1,1,Tbig);
ct=zeros(Tbig,cB+1);
Kapt=NaN(cB+1,cB+1,Tbig); %does the construction of Kapt, Rt, Qt go faster?
Rt=Kapt;
Qt=Kapt;
Qmat=(diag(optpars(cA+1:end-1)))^2; % by 'coincidence' this is the same as with the simple WL_tvreg, because the vorletzte element IS include (sigma_eps), but not the last (the rho)
for tt=1:Tbig;
    Kapt(:,:,tt)=eye(cB+1); Kapt(cB+1,cB+1,tt)=optpars(end); 
    Rt(:,:,tt)=eye(cB+1);
    Qt(:,:,tt)=Qmat;
end;
inista=1; % use the given a0, P0
istv=1;
[afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);    
% now run smoother
[asms,Psms]=WL_Kalmansmo(afis,Pfis,aprs,Pprs,Kapt,istv);


%____________________________________________________________________
%   Creation of further results
%____________________________________________________________________
nontvparhat=optpars(1:cA);
RWstdhat=optpars(cA+1:end-2);
nontvparse=NaN(cA,1);
RWstdse=NaN(cB,1);
parfilt=afis;
parsmo=asms;
parfiltVari=Pfis;
parsmoVari=Psms;
stdepshat=optpars(end-1);
rhohat=optpars(end);

%return;
%____________________________________________________________________
%   Nested function: objective function for ML estimation
%____________________________________________________________________
    function negllh=objfctloc(pars,a0,P0,y,XA,XB)
    % set up state space model
    yt=y;
    P0(end,end)=(pars(end-1)^2)/(1-pars(end)^2);
    [Tbig,cA]=size(XA);
    [Tbig,cB]=size(XB);
    gam=pars(1:cA);
    dt=XA*gam;
    XB1=[XB ones(Tbig,1)]; % the ones are needed to pick the error as last state vector element
    Zt=reshape(XB1',1,cB+1,Tbig); % WL_Kalman requires this to be an array
    Ht=reshape(zeros(Tbig,1),1,1,Tbig); % with the AR(1) error there is no meas error
    ct=zeros(Tbig,cB+1);
    Kapt=NaN(cB+1,cB+1,Tbig); %does the construction of Kapt, Rt, Qt go faster?
    Rt=Kapt;
    Qt=Kapt;
    Qmat=(diag(pars(cA+1:end-1)))^2; % by 'coincidence' this is the same as with the simple WL_tvreg, because the vorletzte element IS include (sigma_eps), but not the last (the rho)
    for t=1:Tbig;
        Kapt(:,:,t)=eye(cB+1); Kapt(cB+1,cB+1,t)=pars(end); 
        Rt(:,:,t)=eye(cB+1);
        Qt(:,:,t)=Qmat;
    end;
    inista=1; % use the given a0, P0
    istv=1;
    [afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);
    negllh=-llh;
    end %end nested function

end %main function

    