% Wolfgang Lemke Nov 2010.
% % same as WL_tvreg but with AR(2) error
% TO BE DONE: join all these functions together

function [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,parfilt,parsmo,parfiltVari,parsmoVari,Xnew,stdepsini,rhohat,yprs,Fprs]=WL_tvreg_AR2error(y,X,tvind,a0,P0,P0scale,RWstdini,nontvini,epstol,optalgo)

% Model is
% y_t  = XA_t'beta_const  + XB_t'beta_t + u_t
% u_t = rho_1 u_{t-1} + rho_2 u_{t-2} + eps_t, eps_t \sim N(0,sigma^2)

% INPUT
%... complete

% OUTPUT:
% ...
% stdepshat: this now the ML estimate of sigma
% 
% TO DO

%____________________________________________________________________
%   Partition X
%____________________________________________________________________
% Resort the columns of X, s.t. first the variables, for which regression
% coefficients are constant, appear, and then the others
% Xnew = [X^A X^B];
[Tbig,cX]=size(X);
allcol=(1:cX)';
nontvind=(tvind-1).^2;
nontvcol=selif(allcol,nontvind); tvcol=selif(allcol,tvind);
XA=X(:,nontvcol); XB=X(:,tvcol);
[Tbig,cXA]=size(XA); [Tbig,cXB]=size(XB); 
Xnew=[XA XB];

%____________________________________________________________________
%   Constant param regression and starting values
%____________________________________________________________________
% Run constant param regression to get
% i) starting values for the ML estimates of constant parameters
% ii) starting values for the  states representing the tv parameters a0
% iii) the variance of the latter P0
% ... unless they are given over as input to the function directly 
% iv) the variance of the regression innovation
[betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y,Xnew);
if nontvini==0;
   nontvini=betahat(1:cXA);
end; 
if isnan(a0);
   a0=[betahat(cXA+1:end); zeros(2,1)];
end;
if isnan(P0);
   P0=zeros(length(a0),length(a0)); 
   P0(1:end-2,1:end-2)=P0scale*Varbetahat(cXA+1:end,cXA+1:end);
else;
   P0=P0*P0scale;
end;
% NOTE: the lower-right elements of P0 will be overwritten anyway in the function

% estimate an Ar2 on the OLS residuals
[rho_u,tval_u,resid_u,Rsq_u,Varbetahat_u]=WL_ols(resid(3:end),[resid(2:end-1) resid(1:end-2)]);

stdepsini=std(resid_u);
start_pars=[nontvini; RWstdini; stdepsini; rho_u];

%____________________________________________________________________
%   Maximize likelihood
%____________________________________________________________________
%     
if optalgo==1; %simplex
    mytol=epstol;
    mymaxiters=1500;
    options = optimset('Display','iter','MaxFunEvals',9000000,'MaxIter',mymaxiters,'TolFun',mytol,'TolX',mytol,'TolCon',mytol,'HessUpdate','bfgs');
    optpars=fminsearch(@(pars) objfctloc(pars,a0,P0,y,XA,XB),start_pars,options);
elseif optalgo==2; % BFGS
    mymaxitersBFGS=150;
    mytolBFGS=epstol;
    optionsBFGS=optimset('Display','iter','MaxFunEvals',100000,'MaxIter',mymaxitersBFGS,'TolFun',mytolBFGS,'TolX',mytolBFGS,'TolCon',mytolBFGS,'HessUpdate','bfgs');
    [optpars,neglllh,exflag]=fminunc(@(pars)objfctloc(pars,a0,P0,y,XA,XB),start_pars,optionsBFGS);
end; %
% 
%optpars=start_pars;
%____________________________________________________________________
%   Filtering given optimal parameters
%____________________________________________________________________
yt=y;
% compute the VarCov-matrix of the AR(2) error part:
rho1=optpars(end-1);
rho2=optpars(end);
sigAR2inno=optpars(end-2);
VarAR2=((1-rho2)*sigAR2inno^2)/((1+rho2)*((1-rho2)^2-rho1^2));
AutoCovAR2=rho1/(1-rho2)*VarAR2;
P0AR2=[VarAR2 AutoCovAR2; AutoCovAR2 VarAR2];

P0(end-1:end,end-1:end)=P0AR2;
[Tbig,cA]=size(XA);
[Tbig,cB]=size(XB);
gam=optpars(1:cA);
dt=XA*gam;
XB1=[XB ones(Tbig,1) zeros(Tbig,1)]; % the ones are needed to pick the error as last-but-one state vector element
Zt=reshape(XB1',1,cB+2,Tbig); % WL_Kalman requires this to be an array
Ht=reshape(zeros(Tbig,1),1,1,Tbig);
ct=zeros(Tbig,cB+2);
Kapt=NaN(cB+2,cB+2,Tbig); %does the construction of Kapt, Rt, Qt go faster?
Rt=NaN(cB+2,cB+1,Tbig);
Qt=NaN(cB+1,cB+1,Tbig);
Qmat=(diag(optpars(cA+1:end-2)))^2; 
for tt=1:Tbig;
    Kapt(:,:,tt)=eye(cB+2); Kapt(end-1:end,end-1:end,tt)=[rho1 rho2; 1 0]; 
    Rt(:,:,tt)=[eye(cB+1); zeros(1,cB+1)];
    Qt(:,:,tt)=Qmat;
end;
inista=1; % use the given a0, P0
istv=1;
[afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);    
% now run smoother
[asms,Psms]=WL_Kalmansmo(afis,Pfis,aprs,Pprs,Kapt,istv);


%____________________________________________________________________
%   Creation of further results
%____________________________________________________________________
nontvparhat=optpars(1:cA);
RWstdhat=optpars(cA+1:end-3);
nontvparse=NaN(cA,1);
RWstdse=NaN(cB,1);
parfilt=afis;
parsmo=asms;
parfiltVari=Pfis;
parsmoVari=Psms;
stdepshat=optpars(end-2);
rhohat=optpars(end-1:end);

%return;
%____________________________________________________________________
%   Nested function: objective function for ML estimation
%____________________________________________________________________
    function negllh=objfctloc(pars,a0,P0,y,XA,XB)
        % set up state space model
        yt=y;
        % compute the VarCov-matrix of the AR(2) error part:
        rho1=pars(end-1);
        rho2=pars(end);
        sigAR2inno=pars(end-2);
        VarAR2=((1-rho2)*sigAR2inno^2)/((1+rho2)*((1-rho2)^2-rho1^2));
        AutoCovAR2=rho1/(1-rho2)*VarAR2;
        P0AR2=[VarAR2 AutoCovAR2; AutoCovAR2 VarAR2];
        P0(end-1:end,end-1:end)=P0AR2;
        [Tbig,cA]=size(XA);
        [Tbig,cB]=size(XB);
        gam=pars(1:cA);
        dt=XA*gam;
        XB1=[XB ones(Tbig,1) zeros(Tbig,1)]; % the ones are needed to pick the error as last-but-one state vector element
        Zt=reshape(XB1',1,cB+2,Tbig); % WL_Kalman requires this to be an array
        Ht=reshape(zeros(Tbig,1),1,1,Tbig);
        ct=zeros(Tbig,cB+2);
        Kapt=NaN(cB+2,cB+2,Tbig); %does the construction of Kapt, Rt, Qt go faster?
        Rt=NaN(cB+2,cB+1,Tbig);
        Qt=NaN(cB+1,cB+1,Tbig);
        Qmat=(diag(pars(cA+1:end-2)))^2;
        for t=1:Tbig;
            Kapt(:,:,t)=eye(cB+2); Kapt(end-1:end,end-1:end,t)=[rho1 rho2; 1 0];
            Rt(:,:,t)=[eye(cB+1); zeros(1,cB+1)];
            Qt(:,:,t)=Qmat;
        end;
        inista=1; % use the given a0, P0
        istv=1;
        [afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);
        negllh=-llh;
    end %end nested function

end %main function

    