% Wolfgang Lemke 
% first version 26 Apr 2010.
% last change 27 April 2010
% function that estimates regression model with tv parameters
% same as WL_tvreg_spv, but 

% - tv vola may be spanned by regressors Z different from X
% - a0, P0 are either given numeric or  as Na
% - tv vola and constant vola are nested: constant vola is a vector of ones
% for Z


function [RWstdhat,nontvparhat,stdepshat,RWstdse,nontvparse,khat,se_khat,parfilt,parsmo,Xnew,stdepsini,a0,P0,MiscOut]=WL_tvreg_spv_sepvolaregr_CISSUS(y,X,tvInd,a0,P0,P0Scale,RWstdini,epstol,optalgo, ksv, Z,volatype)

% INPUT
%... complete
% tvvolaind: vector of zeros and ones, same dim as X, has ones there where
% d elements of tv vola (see above) should be estimated and zeros there,
% where d elements should stay on their starting values (usually zero)
% (note: can generalize later that other exog variables span the vola)
% tvvolasv: starting values for the vola params k (the k0 will be started
% with the ols std dev)

% OUTPUT:
% ...

%____________________________________________________________________
%   Partition X
%____________________________________________________________________
% Resort the columns of X, s.t. first the variables, for which regression
% coefficients are constant, appear, and then the others
% Xnew = [X^A X^B];
[Tbig,cX]=size(X);
allcol=(1:cX)';
nontvInd=(tvInd-1).^2;
nontvcol=selif(allcol,nontvInd); tvcol=selif(allcol,tvInd);
XA=X(:,nontvcol); XB=X(:,tvcol);
[Tbig,cXA]=size(XA); [Tbig,cXB]=size(XB); 
Xnew=[XA XB];

%____________________________________________________________________
%   Constant param regression and starting values
%____________________________________________________________________
% Run constant param regression to get
% i) starting values for the ML estimates of constant parameters
% ii) starting values for the  states representing the tv parameters a0
% iii) the variance of the latter P0
% ... unless they are given over as input to the function directly 
% iv) the variance of the regression innovation
%[betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y,Xnew);
[betahat,tval,resid,Rsq,Varbetahat]=WL_ols(y(1:end-16-1),Xnew(1:end-16-1,:)); % end in 2007 (143) or in 2008Q3 (150)


nontvini=betahat(1:cXA);
if isnan(a0);
   a0=betahat(cXA+1:end);
end;
if isnan(P0);
   P0=P0Scale*Varbetahat(cXA+1:end,cXA+1:end);
else;
   P0=P0*P0Scale;
end;
stdepsini=std(resid);
% UNLIKE FOR WL_tvreg, the variance is differently parametrized:
% i.e. for the homosked version: sigma^2 == exp(k0)
start_pars=[nontvini; RWstdini];
if std(Z)==0; ksv(1)=log((stdepsini)^2); end;
start_pars=[start_pars; ksv];

%____________________________________________________________________
%   Maximize likelihood
%____________________________________________________________________
if optalgo==1; %simplex
    mytol=epstol;
    mymaxiters=1500;
    options = optimset('Display','iter','MaxFunEvals',9000000,'MaxIter',mymaxiters,'TolFun',mytol,'TolX',mytol,'TolCon',mytol,'HessUpdate','bfgs');
    optpars=fminsearch(@(pars) objfctloc(pars,a0,P0,y,XA,XB,ksv,Z,volatype),start_pars,options);
elseif optalgo==2; % BFGS
    mymaxitersBFGS=1500;
    mytolBFGS=epstol;
    optionsBFGS=optimset('Display','iter','MaxFunEvals',100000,'MaxIter',mymaxitersBFGS,'TolFun',mytolBFGS,'TolX',mytolBFGS,'TolCon',mytolBFGS,'HessUpdate','bfgs');
    [optpars,neglllh,exflag,MYOUTPUT,MYRAD,MYHESSIAN]=fminunc(@(pars)objfctloc(pars,a0,P0,y,XA,XB,ksv,Z,volatype),start_pars,optionsBFGS);
    COV_optpars=inv(MYHESSIAN);
    SE_optpars=sqrt(diag(COV_optpars));
end; %
% 
%optpars=start_pars;
%____________________________________________________________________
%   Filtering given optimal parameters
%____________________________________________________________________
yt=y;
[Tbig,cA]=size(XA);
[Tbig,cB]=size(XB);
Xresort=[XA XB];
gam=optpars(1:cA);
dt=XA*gam;
Zt=reshape(XB',1,cB,Tbig); % WL_Kalman requires this to be an array

k=optpars(cA+cB+1:end);

Ht=NaN(1,1,Tbig);
ct=zeros(Tbig,cB);
Kapt=NaN(cB,cB,Tbig); %does the construction of Kapt, Rt, Qt go faster?
Rt=Kapt;
Qt=Kapt;
Qmat=(diag(optpars(cA+1:cA+cB)))^2;
for tt=1:Tbig;
        if volatype==0;
            Ht(:,:,tt)=exp(Z(tt,:)*k);
        elseif volatype==1;
            Ht(:,:,tt)=(Z(tt,:).^2)*(k.^2);
        elseif volatype==2;
            Ht(:,:,tt)=(Z(tt,:))*(k.^2);
        end;
    Kapt(:,:,tt)=eye(cB);
    Rt(:,:,tt)=eye(cB);
    Qt(:,:,tt)=Qmat;
end;
inista=1; % use the given a0, P0
istv=1;
[afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);    
% now run smoother
[asms,Psms]=WL_Kalmansmo(afis,Pfis,aprs,Pprs,Kapt,istv);


%____________________________________________________________________
%   Creation of further results
%____________________________________________________________________
nontvparhat=optpars(1:cA);
RWstdhat=optpars(cA+1:cA+cB);
khat=[];
se_khat=[];
if length(optpars)>cA+cB;
    khat=optpars(cA+cB+1:end);
    se_khat=SE_optpars(cA+cB+1:end);
end;
nontvparse=NaN(cA,1);
RWstdse=NaN(cB,1);
parfilt=afis;
parsmo=asms;
stdepshat=NaN(Tbig,1); % this is now a series!
for tt=1:Tbig;
    if volatype==0;
        stdepshat(tt)=sqrt(exp(Z(tt,:)*k));
    elseif volatype==1;
        stdepshat(tt)=sqrt((Z(tt,:).^2)*(k.^2));
    elseif volatype==2;
        stdepshat(tt)=sqrt(Z(tt,:)*(k.^2));
    end;
end;

MiscOut=5555;

%return;
%____________________________________________________________________
%   Nested function: objective function for ML estimation
%____________________________________________________________________
    function negllh=objfctloc(pars,a0,P0,y,XA,XB,ksv,Z,volatype)
    % set up state space model
    yt=y;
    Xresort=[XA XB];
    [Tbig,cA]=size(XA);
    [Tbig,cB]=size(XB);
    gam=pars(1:cA);
    dt=XA*gam;
    Zt=reshape(XB',1,cB,Tbig); % WL_Kalman requires this to be an array
    
    Ht=zeros(1,1,Tbig); % this array structure required for the WL_Kalman routine
    ct=zeros(Tbig,cB);
    Kapt=NaN(cB,cB,Tbig); %does the construction of Kapt, Rt, Qt go faster?
    Rt=Kapt;
    Qt=Kapt;
    Qmat=(diag(pars(cA+1:cA+cB)))^2;

    k=pars(cA+cB+1:end);
        
    for tc=1:Tbig;
        if volatype==0;
            Ht(:,:,tc)=exp(Z(tc,:)*k);
        elseif volatype==1;
            Ht(:,:,tc)=(Z(tc,:).^2)*(k.^2);
        elseif volatype==2;
            Ht(:,:,tc)=(Z(tc,:))*(k.^2);
        end;
        Kapt(:,:,tc)=eye(cB);
        Rt(:,:,tc)=eye(cB);
        Qt(:,:,tc)=Qmat;
    end;
    inista=1; % use the given a0, P0
    istv=1;
    [afis,Pfis,yprs,Fprs,aprs,Pprs,llh]=WL_Kalman(yt,dt,Zt,Ht,ct,Kapt,Rt,Qt,a0,P0,inista,istv);
    negllh=-llh;
    end %end nested function

end %main function

    