function [X,WW,MM,nobs,nv,id,codes,varnames,Nsub0, pos_fci] = dataload(v, obs, opt_endtime, opt_starttime, sheetname, opt_fci, wantblockexogobs)
 
%inputs
    %optionper: 0/1/2 period 0 (ab 1976)/1 (1976-90)/2 (ab 1991)
    %obs: observables
    %v: variables of interest
    %countr: which countries we'd like to include in our dataset (the
    %entire dataset contains other OECD countries)
%output
    %X: transformed data (TxN)
    %WW: stddev
    %MM: mean
    %nv: position of variables of interest in dataset
    %nobs: position of observable(s) in dataset
    %id: group - position in group as in xls file - position in dataset
    %codes: codes for data transformation (later used to cumulate IRFs)
    %varnames: country names
    %weights: tv gdp weights

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Load data

if  wantblockexogobs == 0
    data_range     = 'c9:k182'; 
else
    data_range     = 'd9:k182'; % Do not load US data 
end

if strcmp(opt_fci ,'FCI_Watson') % Hatzius et al. FCI
    data_range_fci  = 'e9:e182';   
%===========================================================================    
elseif strcmp(opt_fci ,'FCI_Watson_purged')% Hatzius et al. FCI - macro avriables removed
    data_range_fci  = 'h9:h182'; 
%===========================================================================        
elseif strcmp(opt_fci ,'FCI_International') % International FCI 
    data_range_fci  = 'f9:f182';    
%===========================================================================    
elseif strcmp(opt_fci ,'FSI')   % FSI Hubrich and Tetlow
    data_range_fci  = 'g9:g182';       
end


%=== Load data
gdp = xlsread(sheetname,'GDPreal',data_range); %1
cons = xlsread(sheetname,'Consreal',data_range); %2
inv = xlsread(sheetname,'GFCFreal',data_range); %3
residinv = xlsread(sheetname,'Residinvreal',data_range); %4
nonresidinv = xlsread(sheetname,'Nonresidinvreal',data_range); %5
govcons = xlsread(sheetname,'Govconsreal',data_range);
govd = xlsread(sheetname,'Govdeficit',data_range); %7
ex = xlsread(sheetname,'EXreal',data_range);
im = xlsread(sheetname,'IMreal',data_range);
ip = xlsread(sheetname,'IP',data_range);
unempl = xlsread(sheetname,'Unemplrate',data_range);
tfp = xlsread(sheetname,'TFP',data_range);
cpi = xlsread(sheetname,'CPI',data_range);
gdpdefl = xlsread(sheetname,'GDPdefl',data_range);
exp = xlsread(sheetname,'EXp',data_range);
imp = xlsread(sheetname,'IMp',data_range);
equityp = xlsread(sheetname,'Equityp',data_range); %17
housep = xlsread(sheetname,'Housep',data_range); %18
credit = xlsread(sheetname,'Creditpriv',data_range); %19
stint = xlsread(sheetname,'STint',data_range); %20
ltint = xlsread(sheetname,'LTint',data_range); %21
reer = xlsread(sheetname,'REER',data_range);   %22
erusd = xlsread(sheetname,'ERUSD',data_range); %23

%convert equity p, house p, credit in real by dividing by the GDP deflator
P = gdpdefl(3:end,:);
equityp(3:end,:) = equityp(3:end,:)./P;
housep(3:end,:) = housep(3:end,:)./P;
credit(3:end,:) = credit(3:end,:)./P;

[~,varnames] = xlsread(sheetname,'REER','c7:u7');

% Load FCI
if  wantblockexogobs == 0
    observ  = xlsread(sheetname,'FCI',data_range_fci); %17
    pos_fci = 1;
else
    fci     = xlsread(sheetname,'FCI',data_range_fci);  
    usgdp   = xlsread(sheetname,'GDPreal','c9:c182');  
    usp     = xlsread(sheetname,'GDPdefl','c9:c182');
    usffr   = xlsread(sheetname,'STint','c9:c182');
    observ = [usgdp usp usffr fci];
    pos_fci = 4;
end

 
% Get how many countries for each series
Nsub = ones(24,1).*(9-wantblockexogobs);     % 9 countries for each of the 23 variables
Nsub(end)=size(observ,2); % one FCI

%__________________________________________________________________________
%stack all data
X = [gdp cons inv residinv nonresidinv govcons ex im ip unempl gdpdefl cpi exp imp equityp housep credit stint ltint reer erusd govd tfp observ];

codes = X(1,:); %codes for data transformation
X = X(3:end,:); %data start in 1970Q1

%__________________________________________________________________________
%period
 
if strcmp(opt_starttime,'1971')
    X = X(5:end,:);%(5:end,:); %ab 71Q1
elseif strcmp(opt_starttime,'1973')
    X = X(13:end,:);%(5:end,:); %ab 73Q1
elseif strcmp(opt_starttime,'1975')
    X = X(13+8:end,:);%(5:end,:); %ab 73Q1    
end

[T,N] = size(X);

% weights = [];
% for i = 1:length(countr)
%     weights(:,i) = X(:,i)./sum(X(:,1:length(countr)),2);
% end

nb = 1:N;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%Remove series with missings and adjust X, codes, nb accordingly
X0 = []; nb0 = []; codes0 = []; 
for i = 1:N
    j = 0;
    for t = 1:T
        if isnan(X(t,i)) == 1, j = 1; break, end
    end
    if j == 0, 
        X0 = [X0 X(:,i)]; 
        nb0 = [nb0 nb(:,i)]; % tells you which of the original series is still there
        codes0 = [codes0 codes(i)];
    end
end

[T,N] = size(X0); % update data size

% Update how many countries for each series
nbtmp = zeros(1,length(nb));
nbtmp(:,nb0) = 1; % place 1, if we have the series
j = 1; Nsub0 = [];
for i = 1:length(Nsub)
    Nsub0(i) = sum(nbtmp(j:j+Nsub(i)-1));
    j = j + Nsub(i);
end

disp('size of dataset')
disp('total')
disp([T N])
disp('gdp cons inv residinv nonresidinv govcons ex im ip unempl gdpdefl cpi exp imp equityp housep credit stint ltint reer erusd govd tfp observ')
disp(Nsub0)

X = X0;
codes = codes0; clear codes0; 
%==========================================================================
%=== Transform the data
%==========================================================================
% Xtemp = X; 
% X = one_sided_hp_filter_kalman(Xtemp);


X_stat = transfstat(X,codes); %transform to have stationary data

%Removal of possibly shifted means using the Bai and Perron tests prior to fitting  a factor model to the data
% X_statdem = []; mm = [];
%     for i = 1:cols(X_stat),
%         %i, %pause
%         [nbr,brpt,mm(:,i),ydem,bb] = break0(X_stat(:,i));
%         X_statdem = [X_statdem ydem];
%     end
% X_stat = X_statdem;


[X_stato] = outlierSW2005(X_stat,6); %outlier removal
%    X_stato = X_stat;

%[MM,WW,X_statost] = transfstandard_MeanAdj(X_stato); %variance = 1, mean = 0, WW: standard deviations (for IRFs)
[MM,WW,X_statost] = transfstandard(X_stato); %variance = 1, mean = 0, WW: standard deviations (for IRFs)

%==========================================================================
% plot data and construct id (which data are where in the dataset)
id = [];
[T,N] = size(X); % already there, should remove this
j = 1; k = 0;
for i = 1:N
    if i <= sum(Nsub0(1:j)),
        k = k+1;
    else
        j = j+1;
        k = 1;
    end
    
    if j == 1,
        k0 = nb0(i);
    else
        k0 = nb0(i) - sum(Nsub(1:j-1)) ;
    end

 
    
    %[j i k k0 sum(Nsub0(1:j))], x, pause  
     id = [id; j k0 i]; %to identify variables
     
%      disp('total sub')
%      x
%      [nb0(i) k0]
%      subplot(2,1,1), plot(X(:,i)), axis tight,
%      subplot(2,1,2), plot(X_statost(:,i)), axis tight,
%      hold off, pause   

end

%== id has 3 columns: variable index (i.e. 1=GDP), countries for which we
%have the obs. on this variable, and progressive number in teh dataset
% so first variable is US GDP, 9th is Australian GDP

X = X_statost;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%find series of interest (observable(s) and variables for which IRFs will be computed)

%observable(s)
obs = [obs NaN*ones(cols(obs'),1)];  %give j and k0

%variables of interest
v = [v NaN*ones(cols(v'),1)]; % add a column of NaN
  
for i = 1:N
  for ii = 1:cols(obs')
    if id(i,1:2) == obs(ii,1:2), 
        obs(ii,:) = id(i,:); 
    end
  end
  for ii = 1:cols(v')
    if id(i,1:2) == v(ii,1:2), 
        v(ii,:) = id(i,:); 
    end
  end  
end %3rd colums of v and observ. give position of these variables in X

nobs = obs(:,3); 
nv = v(:,3);