%em.m
function [X_hat] = em(Xmiss,r,crit);

%inputs
    %r: nb of static factors
    %Xmiss: unbalanced panel with NaNs for missing values
%output
    %estimated balanced panel

    
if nargin == 2, crit = 0.000001; end
[T,N] = size(Xmiss);

%take balanced panel to get starting values for F
Xbal = [];
for i = 1:N
    tmp = 0;
    for t = 1:T
        if isnan(Xmiss(t,i)) == 1, tmp = 1; end
    end
    if tmp == 0, Xbal = [Xbal Xmiss(:,i)]; end    
end
[T,Nbal] = size(Xbal);
c = Xbal'*Xbal/T;
[evec,lam] = eigrs2(c);
lam = evec(:,Nbal-r+1:Nbal);   
F0 = Xbal*lam;

X_hat0 = []; lam0 = [];
for i = 1:N
    for t = 1:T
        if isnan(Xmiss(t,i)) ~= 1, 
            X_hat0(t,i) = Xmiss(t,i);
        else
            X_hat0(t,i) = 0;
        end
    end
    res = ols(X_hat0(:,i),F0);
    lam0(i,:) = res.beta'; clear res;
end

%minimizing the function
V0 = 1000; c = 1000;
while c > crit
    V = 0;
    X_hat = X_hat0;
    for i = 1:N
        for t = 1:T
            if isnan(Xmiss(t,i)) ~= 1, 
                X_hat(t,i) = Xmiss(t,i);
            else
                X_hat(t,i) = F0(t,:)*lam0(i,:)';
            end
        end
    end
    c = X_hat'*X_hat/T;
    [evec,lam] = eigrs2(c);
    lam = evec(:,N-r+1:N);   
    F = X_hat*lam;
    
    V = sum(sum(X_hat(t,:) - F(t,:)*lam')^2);
    c = abs(V-V0)

    V0 = V;
    F0 = F;
    lam0 = lam;
    X_hat0 = X_hat;
%    figure(6)
%    subplot(2,1,1)
%    plot(X(:,2),'b'), hold on, plot(X_hat(:,2),'r'), hold on, 
%    subplot(2,1,2)
%    plot(X(:,20),'b'), hold on, pause,plot(X_hat(:,20),'r'), hold on, pause
%figure(10),
%plot(F), hold on, pause
end


