function [res,Glb,Gub,epslb,epsub] = latobs(Y,r,G)%,crit)

%inputs
%   Y: TxN dataset
%   r: nb of static factors
%   G: observed factor
%   crit: critical value (0.025)

%if nargin == 3, crit = 0.025; end

[T,N] = size(Y);

%Estimating the common factors    
    [v,d] = eigs(Y*Y'/N/T,r);

    %flip the eigenvectors of cov(X) to preserve signs
    for i = 1:r
        if v(1,i)<0, v(:,i) = -v(:,i); end
    end
    if d(1,1)<d(r,r)
        d = diag(sort(diag(d*(-1)))*(-1));
        v = flipud(v')';
    end    

    F = v*sqrt(T);
    C = Y'*F/T;
        
    CC = F*C';     
    IC = Y - CC;          

    %Testing G(t) one at a time
      %Exact tests
        %A(j)
                %cross-sectional correlated errors, no heteroscedast. (relevant case)
                n = round(min(sqrt(N),sqrt(T))); %faustregel von BN 2004 ("confidence intervals ...")
                sum = zeros(r,r);
                for i = 1:n
                        for j = 1:n
                            sum = sum + C(i,:)'*C(j,:)*mean(IC(:,i).*IC(:,j));
                        end
                end
                K = sum/n;
                
                res = ols(G,F);
                gam = res.beta;
                G_hat = F*res.beta;
                varG = 1/N * gam' * inv(d) * K * inv(d) * gam; 
                tau = (G_hat - G)/sqrt(varG);

                A = 0;
                for t = 1:T
                    if abs(tau(t,:)) > 1.96, 
                        A = A + 1; 
                    end
                end
                A = A/T; %frequency that abs(tau) exceeds the 5% asymptotical crit. value
        %M(j)
                M = max(abs(tau));  %krit. Wert fuer T = 400: 3.830
                
     %Approximate tests
     
                Glb = G_hat - 1.96*sqrt(varG);
                Gub = G_hat + 1.96*sqrt(varG);
                
                eps = G - G_hat;   
                s = [];
                for t = 1:T
                    s(t,1) = sqrt(1/T*F(t,:)*F(t,:)'*var(eps) + varG); %E homoscedast. ??? why always 1???
                end    
                epslb = eps - 1.96*s; 
                epsub = eps + 1.96*s;  
                
                NS = var(eps)/var(G_hat);%var(eps)/varG  %check!
                R2 = var(G_hat)/var(G);% same as res.rsqr %varG/var(G) %check!
 
                CIeps = 0;
                for t = 1:T
                    if (epslb(t,:) < 0)&(epsub(t,:) >0), CIeps = CIeps + 1; end
                end
                CIeps = CIeps/T;
%Testing G as a set
%               [T,m] = size(G);

%               min_cc = []; min_cc_CI = [];
%               K0 = [];
%               S_FF = F'*F; S_GG = G'*G; S_FG = F'*G; S_GF = G'*F;  %canonical correlations
%               KK = inv(S_FF)*S_FG*inv(S_GG)*S_GF;
%               AA = inv(S_GG)*S_GF*inv(S_FF)*S_FG;
%               [K0,dKK] = eigs(KK,min(r,m));
%               [AAA,dAA] = eigs(AA,min(r,m));
%               for i = 1:min(r,m)     %normalizing to get A'*S_GG*A=I and K'*S_FF*K=I
%                    K0(:,i) = K0(:,i)/sqrt(K0(:,i)'*S_FF*K0(:,i));
%                    AAA(:,i) = AAA(:,i)/sqrt(AAA(:,i)'*S_GG*AAA(:,i));
%               end   
%               R = AAA'*S_GF*K0; %can be neg.; Proposition 20.1. in Hamilton (p. 632) only ensures that square = square of canonical correl.
%               min_cc = min(diag(abs(R)));  %report the min. canonical correlation coef.
%               min_cc2 = min_cc^2;
%               min_cc2_CI = [min_cc2 - 2*1.96*min_cc*(1-min_cc2)/sqrt(T) min_cc2 + 2*1.96*min_cc*(1-min_cc2)/sqrt(T)];
      
%               disp('Smallest canonical correlation coefficient between G and F_hat and confidence bands')
%               [min_cc2 min_cc2_CI]  

%disp('A M R2 NS CIeps min_cc2 min_cc2_CI')

res = [A M R2 NS CIeps];% min_cc2 min_cc2_CI];

%figure(2), 
%subplot(2,1,1), plot([Glb Gub],'k:'), hold on, plot(G,'r')                
%subplot(2,1,2), plot([epsub epslb],'k'), hold on, plot(zeros(T,1),'r')
                

